<?php

	/**
	 * MiPP connector class.
	 */
	class MiPP
	{
		/**
		 * The api key to use.
		 */
		private $apiKey;

		/**
		 * The api url to use.
		 */
		private $url = 'https://pay.myappy.it/api/v1';

		/**
		 * Constructor. Takes the api key used for authorization.
		 */
		function __construct($apiKey)
		{
			$this->apiKey = $apiKey;
		}

		/**
		 * Retrieves a payment.
		 */
		function getPayment($uid)
		{
			$res = NULL;
			$headers = array
			(
				'Authorization: Bearer ' . $this->apiKey,
				'Content-Type: application/json'
			);

			$url = $this->url . '/payment/' . urlencode($uid);

			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, $url);
			curl_setopt($ch, CURLOPT_POST, FALSE);
			curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
			curl_setopt ($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
			$result = curl_exec($ch);

			if ($result !== FALSE && ($json = json_decode($result, TRUE)) != NULL)
			{
				$res = $json;
			}
			else if ($result === FALSE)
			{
				$res = array('success' => FALSE, 'error' => curl_error($ch));
			}
			else
			{
				$res = array('success' => FALSE, 'error' => json_last_error_msg(), 'debug' => $result, 'url' => $url);
			}

			curl_close($ch);

			return $res;
		}

		/**
		 * Retrieves user information.
		 */
		function getInfo()
		{
			$res = NULL;
			$headers = array
			(
				'Authorization: Bearer ' . $this->apiKey,
				'Content-Type: application/json'
			);

			$url = $this->url . '/user';

			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, $url);
			curl_setopt($ch, CURLOPT_POST, FALSE);
			curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
			curl_setopt ($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
			$result = curl_exec($ch);

			if ($result !== FALSE && ($json = json_decode($result, TRUE)) != NULL)
			{
				$res = $json;
			}
			else if ($result === FALSE)
			{
				$res = array('success' => FALSE, 'error' => curl_error($ch));
			}
			else
			{
				$res = array('success' => FALSE, 'error' => json_last_error_msg(), 'debug' => $result, 'url' => $url);
			}

			curl_close($ch);

			return $res;
		}

		/**
		 * Initiates a payment. Required fields in data array: amount, description, callback.
		 * Optional fields: auth_only, public_url, metadata
		 */
		function postPayment($data)
		{
			$res = NULL;
			$headers = array
			(
				'Authorization: Bearer ' . $this->apiKey,
				'Content-Type: application/json'
			);

			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, $this->url . '/payment');
			curl_setopt($ch, CURLOPT_POST, TRUE);
			curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
			curl_setopt ($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
			curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
			$result = curl_exec($ch);

			if ($result !== FALSE && ($json = json_decode($result, TRUE)) != NULL)
			{
				$res = $json;
			}
			else if ($result === FALSE)
			{
				$res = array('success' => FALSE, 'error' => curl_error($ch));
			}
			else
			{
				$res = array('success' => FALSE, 'error' => json_last_error_msg());
			}

			curl_close($ch);

			return $res;
		}

		/**
		 * Updates a payment.
		 */
		function updatePayment($uid, $data)
		{
			$res = NULL;
			$headers = array
			(
				'Authorization: Bearer ' . $this->apiKey,
				'Content-Type: application/json'
			);

			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, $this->url . '/payment/' . $uid);
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'PUT');
			curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
			curl_setopt ($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
			curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
			$result = curl_exec($ch);

			if ($result !== FALSE && ($json = json_decode($result, TRUE)) != NULL)
			{
				$res = $json;
			}
			else if ($result === FALSE)
			{
				$res = array('success' => FALSE, 'error' => curl_error($ch));
			}
			else
			{
				$res = array('success' => FALSE, 'error' => json_last_error_msg());
			}

			curl_close($ch);

			return $res;
		}
	}

?>
