<?php
/**
 * Plugin Name: Car Rental Software - OTA Booking Engine
 * Plugin URI: http://www.car-rental-software.it
 * Description: Car Rental Software - OTA Booking Engine Plugin
 * Author: My Appy
 * Author URI: http://www.myappy.it/
 * Version: 1.9
 * Text Domain: otabo
 *
 * Copyright: (c) 2020-2022 My Appy (info@myappy.net)
 *
 * @author    My Appy
 * @copyright Copyright (c) 2020-2022, My Appy
 *
 */

// define('DEBUG', 1);

class Otabo
{
	function __construct($locale = NULL)
	{
		// Set the default colors.
		if (!get_option('otabo_hue') || get_option('otabo_hue') == '')
		{
			if (get_option('otabo_hue') === NULL)
			{
				add_option('otabo_hue', '120');
			}
			else
			{
				update_option('otabo_hue', '120');
			}
		}

		if (!get_option('otabo_sat') || get_option('otabo_sat') == '')
		{
			if (get_option('otabo_sat') === NULL)
			{
				add_option('otabo_sat', '100%');
			}
			else
			{
				update_option('otabo_sat', '100%');
			}
		}

		if (!get_option('otabo_lig') || get_option('otabo_lig') == '')
		{
			if (get_option('otabo_lig') === NULL)
			{
				add_option('otabo_lig', '35%');
			}
			else
			{
				update_option('otabo_lig', '35%');
			}
		}

		if (!get_option('otabo_booking_fields') || get_option('otabo_booking_fields') == '')
		{
			if (get_option('otabo_booking_fields') === NULL)
			{
				add_option('otabo_booking_fields', 'name,surname,email,email_confirm,phone,address,city,zip_code,country');
			}
			else
			{
				update_option('otabo_booking_fields', 'name,surname,email,email_confirm,phone,address,city,zip_code,country');
			}
		}

		if (!get_option('otabo_optionals_groups') || get_option('otabo_optionals_groups') == '' || json_decode(get_option('otabo_optionals_groups'), TRUE) == NULL)
		{
			if (get_option('otabo_optionals_groups') === NULL)
			{
				add_option('otabo_optionals_groups', json_encode($this->OPTIONAL_GROUPS));
			}
			else
			{
				update_option('otabo_optionals_groups', json_encode($this->OPTIONAL_GROUPS));
			}
		}

		if (get_option('otabo_form_one_way_trips', NULL) === NULL)
		{
			add_option('otabo_form_one_way_trips', 1);
		}

		$this->OPTIONAL_GROUPS = json_decode(get_option('otabo_optionals_groups'), TRUE);

		$osd = isset($_POST['osd']) ? $_POST['osd'] : (isset($_GET['osd']) ? $_GET['osd'] : '');
		$siteUrl = get_site_url();

		define('_SG', 'OTABO:' . sha1($siteUrl));
		define('_SI', 'OTABO:' . sha1($siteUrl) . ($osd != '' ? '-' . $osd : ''));

		$defLang = array_keys($this->STRINGS);
		$defLang = array_shift($defLang);

		$lang = isset($_GET['otabo_lang']) && isset($this->STRINGS[$_GET['otabo_lang']]) ? $_GET['otabo_lang'] : NULL;
		if ($lang == NULL && $locale)
		{
			if (($p = strpos($locale, '_')) !== FALSE)
			{
				$locale = substr($locale, 0, $p);
			}

			if (($p = strpos($locale, '-')) !== FALSE)
			{
				$locale = substr($locale, 0, $p);
			}

			if (isset($this->STRINGS[$locale]))
			{
				$lang = $locale;
			}
		}

		if (!$lang)
		{
			$lang = $defLang;
		}

		$customPhrases = get_option('otabo_custom_phrases');
		if ($customPhrases && strlen($customPhrases) > 0 && ($customPhrases = json_decode($customPhrases, TRUE)) != NULL
				&& isset($customPhrases[$lang]))
		{
			foreach ($customPhrases[$lang] as $k => $v)
			{
				$this->STRINGS[$lang][$k] = $v;
			}
		}

		$this->LANG = $lang;

		$appendix = ($osd != '' ? '?osd=' . $osd : '');

		define('OTABO_OID_SEP', 'x');
		define('OTABO_OID_QTY', '×');

		define('OTABO_FORM_URL', $this->getI18nPageLink(get_option('otabo_form_url')) . $appendix);
		define('OTABO_RESULTS_URL', $this->getI18nPageLink(get_option('otabo_results_url')) . $appendix);
		define('OTABO_CONFIRM_URL', $this->getI18nPageLink(get_option('otabo_confirm_url')) . $appendix);

		if ($appendix != '')
		{
			$appendix .= isset($_GET['qid']) && $_GET['qid'] != '' ? '&qid=' . $_GET['qid'] : '';
			$appendix .= isset($_GET['cid']) && $_GET['cid'] != '' ? '&cid=' . $_GET['cid'] : '';
			$appendix .= isset($_GET['oid']) && $_GET['oid'] != '' ? '&oid=' . $_GET['oid'] : '';
		}

		define('OTABO_BOOKING_COVERAGES_URL', $this->getI18nPageLink(get_option('otabo_coverages_url')) . $appendix);
		define('OTABO_BOOKING_OPTIONALS_URL', $this->getI18nPageLink(get_option('otabo_optionals_url')) . $appendix);
		define('OTABO_BOOKING_URL', $this->getI18nPageLink(get_option('otabo_booking_url')) . $appendix);

		if (get_option('otabo_mipp_api_key') && get_option('otabo_mipp_api_key') != '')
		{
			define('OTABO_PAYMENT', TRUE);
		}
		else if (isset($_SESSION[_SI]) && isset($_SESSION[_SI]['quotes_result']) && isset($_SESSION[_SI]['quotes_result']['quotes'])
				&& array_key_exists('qid', $_GET))
		{
			$quote = NULL;
			$vendor = NULL;

			if (strpos($_GET['qid'], '.') === FALSE)
			{
				$quote = isset($_SESSION[_SI]['quotes_result']['quotes'][$_GET['qid']]) ?
						$_SESSION[_SI]['quotes_result']['quotes'][$_GET['qid']] : NULL;
				$vendor = isset($quote['vendor']) ? $quote['vendor'] : NULL;
			}
			else
			{
				$p = strpos($_GET['qid'], '.');
				$qid = substr($_GET['qid'], 0, $p);
				$o = substr($_GET['qid'], $p + 1);

				$quote = isset($_SESSION[_SI]['quotes_result']['quotes'][$qid])
						&& isset($_SESSION[_SI]['quotes_result']['quotes'][$qid]['options'])
						&& isset($_SESSION[_SI]['quotes_result']['quotes'][$qid]['options'][$o]) ?
						$_SESSION[_SI]['quotes_result']['quotes'][$qid]['options'][$o] : NULL;
				$vendor = $quote && isset($_SESSION[_SI]['quotes_result']['quotes'][$qid]['vendor']) ?
						$_SESSION[_SI]['quotes_result']['quotes'][$qid]['vendor'] : NULL;
			}

			if ($quote && $vendor && get_option('otabo_mipp_api_key_' . $vendor['id']))
			{
				define('OTABO_PAYMENT', TRUE);
			}
			else
			{
				define('OTABO_PAYMENT', FALSE);
			}
		}
		else if (isset($_GET['rid']))
		{
		}
		else
		{
			define('OTABO_PAYMENT', FALSE);
		}

		$this->PARAMS =
		[
			'booking_fields_invoicing' => NULL,
			'form_bases' => NULL,
			'form_classes' => NULL,
			'form_classes_list' => NULL,
			'form_one_way_trips' => NULL,
			'form_overflow_fix' => NULL,
			'form_young_driver' => NULL,
			'result_details' => NULL,
			'result_full_width' => NULL,
			'result_groups' => NULL
		];

		foreach ($this->PARAMS as $k => $v)
		{
			$key = 'otabo_' . $k;
			$v = get_option('otabo_' . $k);

			if ($k == 'form_bases' || $k == 'form_classes_list' || $k == 'result_details')
			{
				$v = array_filter(array_map('trim', explode(',', $v ? trim($v) : '')));
				$v = count($v) > 0 ? $v : NULL;
			}
			else if (strtolower($v) == 'on')
			{
				$v = TRUE;
			}
			else if (strtolower($v) == 'off')
			{
				$v = FALSE;
			}

			$this->PARAMS[$k] = $v;
		}
	}

	function getI18nPageLink($id)
	{
		if (function_exists('pll_get_post'))
		{
			$id = pll_get_post($id, $this->LANG);
		}

		return get_page_link($id);
	}

	function addScripts()
	{
		wp_register_script('otabo-script', plugins_url('/script.js', __FILE__), array('jquery'), NULL);
		wp_enqueue_script('otabo-script');

		wp_register_style('otabo-style', plugins_url('/styles.css', __FILE__), array(), '1.0', 'all');
		wp_enqueue_style('otabo-style');

		$inline = ':root
		{
			--otabo-hue: ' . get_option('otabo_hue') . ';
			--otabo-sat: ' . get_option('otabo_sat') . ';
			--otabo-lig: ' . get_option('otabo_lig') . ';

			--otabo-main: hsl(var(--otabo-hue), var(--otabo-sat), var(--otabo-lig));
			--otabo-dark: hsl(var(--otabo-hue), var(--otabo-sat), calc(var(--otabo-lig) * 0.75));
			--otabo-darker: hsl(var(--otabo-hue), var(--otabo-sat), calc(var(--otabo-lig) * 0.5));
			--otabo-transparent: hsla(var(--otabo-hue), var(--otabo-sat), calc(var(--otabo-lig)), 0.1);
		}';

		wp_add_inline_style('otabo-style', $inline);
	}

	function ajax($params = NULL)
	{
		$error = NULL;
		$res = array('success' => FALSE);

		$noOutput = FALSE;
		if (!$params)
		{
			$params = $_POST;
		}
		else
		{
			$noOutput = TRUE;
		}

		$strings = $this->STRINGS[$this->LANG];//array_map(function($e) { return str_replace('\'', '\\\'', $e); }, $this->STRINGS[OTABO_LANG]);

		define('AUTH_ID', get_option('otabo_uuid'));
		define('AUTH_API_KEY', get_option('otabo_api_key'));

		if (isset($params['cmd']))
		{
			$wsdl = 'https://carrentalsoftware.myappy.it/web/ota/opentravel.wsdl';

			try
			{
				///
				///
				// BASES LIST
				///
				///
				if ($params['cmd'] == 'bases')
				{
					$bases = NULL;
					$classes = NULL;

					if (isset($_SESSION[_SG]) && isset($_SESSION[_SG]['bases']) && count($_SESSION[_SG]['bases']) > 0)
					{
						$bases = $_SESSION[_SG]['bases'];
						$classes = isset($_SESSION[_SG]['classes']) ? $_SESSION[_SG]['classes'] : [];
					}
					else
					{
						$xml = '
							<ns1:OTA_VehLocSearchRQ>
							  <POS>
								<Source>
								  <RequestorID Type="29" ID="' . AUTH_ID . '" MessagePassword="' . htmlspecialchars(AUTH_API_KEY) . '"><CompanyName></CompanyName></RequestorID>
								</Source>
							  </POS>
							</ns1:OTA_VehLocSearchRQ>';

						$client = new SoapClient($wsdl, array('trace' => 1, 'cache_wsdl' => WSDL_CACHE_NONE));
						$result = $client->OTA_VehLocSearch(new SoapVar($xml, XSD_ANYXML));

						$cleanXml = str_ireplace(['SOAP-ENV:', 'SOAP:', 'ns1:'], '', $client->__getLastResponse());
						$result = simplexml_load_string($cleanXml)->Body->OTA_VehLocSearchRS;

						$bases = array();
						$classes = array();

						if ($result && isset($result->Success) && $result->Success)
						{
							if (@count($result->VehMatchedLocs->VehMatchedLoc) > 0)
							{
								foreach ($result->VehMatchedLocs->VehMatchedLoc as $b)
								{
									$name = (string) $b->LocationDetail['Name'];
									if (isset($this->STRINGS[$this->LANG]['booking_base_' . (string) $b->LocationDetail['Code']]))
									{
										$name = $this->STRINGS[$this->LANG]['booking_base_' . (string) $b->LocationDetail['Code']];
									}

									if ($name == '')
									{
										continue;
									}

									// Get the hours.
									$closedDays = [];
									$customDays = [];
									$hours = [[], [], [], [], [], [], []];
									if ($b->LocationDetail->AdditionalInfo && $b->LocationDetail->AdditionalInfo->OperationSchedules
											&& $b->LocationDetail->AdditionalInfo->OperationSchedules->OperationSchedule)
									{
										$days = ['Sun', 'Mon', 'Tue', 'Weds', 'Thur', 'Fri', 'Sat'];
										foreach ($b->LocationDetail->AdditionalInfo->OperationSchedules->OperationSchedule as $s)
										{
											if (isset($s['Start']) && isset($s['End']))
											{
												if ($s->OperationTimes && $s->OperationTimes->OperationTime)
												{
													$date = new DateTime($s['Start']);
													while ($date->format('Y-m-d') <= $s['End'])
													{
														$dateStr = $date->format('Y-m-d');
														$customDays[$dateStr] = [];

														foreach ($s->OperationTimes->OperationTime as $t)
														{
															$h = [(string) $t['Start'], (string) $t['End']];
															$customDays[$dateStr][] = $h;
														}

														$date->add(new DateInterval('P1D'));
													}
												}
												else
												{
													$date = new DateTime($s['Start']);
													while ($date->format('Y-m-d') <= $s['End'])
													{
														$closedDays[] = $date->format('Y-m-d');
														$date->add(new DateInterval('P1D'));
													}
												}
											}
											else
											{
												foreach ($s->OperationTimes->OperationTime as $t)
												{
													$h = [(string) $t['Start'], (string) $t['End']];
													foreach ($days as $k => $d)
													{
														if ($t[$d] && strtolower((string) $t[$d]) == 'true')
														{
															$hours[$k][] = $h;
														}
													}
												}
											}
										}
									}

									foreach ($bases as $k => $base)
									{
										if (strcasecmp($base['name'], $name) == 0)
										{
											$bases[$k]['closed_days'][] = $closedDays;
											$bases[$k]['custom_days'][] = $customDays;

											foreach ($bases[$k]['hours'] as $kk => $vv)
											{
												$bases[$k]['hours'][$kk] = array_merge($bases[$k]['hours'][$kk], $hours[$k]);
											}

											$bases[$k]['uid'][] = (string) $b->LocationDetail['Code'];
											$bases[$k]['uuid'][] = (string) $b->LocationDetail['ExtendedLocationCode'];

											$b = NULL;
										}
									}

									if ($b)
									{
										$bases[] = array
										(
											'uid' => array((string) $b->LocationDetail['Code']),
											'uuid' => array((string) $b->LocationDetail['ExtendedLocationCode']),
											'closed_days' => [$closedDays],
											'custom_days' => [$customDays],
											'hours' => $hours,
											'name' => $name
										);
									}
								}
							}
							else
							{
								$b = $result->VehMatchedLocs->VehMatchedLoc;

								$name = (string) $b->LocationDetail['Name'];
								if (isset($this->STRINGS[$this->LANG]['booking_base_' . (string) $b->LocationDetail['Code']]))
								{
									$name = $this->STRINGS[$this->LANG]['booking_base_' . (string) $b->LocationDetail['Code']];
								}

								if ($name != '')
								{
									// Get the hours.
									$closedDays = [];
									$hours = [[], [], [], [], [], [], []];
									if ($b->AdditionalInfo && $b->AdditionalInfo->OperationSchedules && $b->AdditionalInfo->OperationSchedules->OperationSchedule)
									{
										$days = ['Sun', 'Mon', 'Tue', 'Weds', 'Thur', 'Fri', 'Sat'];
										foreach ($b->AdditionalInfo->OperationSchedules->OperationSchedule as $s)
										{
											if (isset($s['Start']) && isset($s['End']))
											{
												if ($s->OperationTimes && $s->OperationTimes->OperationTime)
												{
													$date = new DateTime($s['Start']);
													while ($date->format('Y-m-d') <= $s['End'])
													{
														$dateStr = $date->format('Y-m-d');
														$customDays[$dateStr] = [];

														foreach ($s->OperationTimes->OperationTime as $t)
														{
															$h = [(string) $t['Start'], (string) $t['End']];
															$customDays[$dateStr][] = $h;
														}

														$date->add(new DateInterval('P1D'));
													}
												}
												else
												{
													$date = new DateTime($s['Start']);
													while ($date->format('Y-m-d') <= $s['End'])
													{
														$closedDays[] = $date->format('Y-m-d');
														$date->add(new DateInterval('P1D'));
													}
												}
											}
											else
											{
												foreach ($s->OperationTimes->OperationTime as $t)
												{
													$h = [$t['Start'], $t['End']];
													foreach ($days as $k => $d)
													{
														if ($t[$d] && strtolower((string) $t[$d]) == 'true')
														{
															$hours[$k][] = $h;
														}
													}
												}
											}
										}
									}

									foreach ($bases as $k => $base)
									{
										if (strcasecmp($base['name'], $name) == 0)
										{
											$bases[$k]['closed_days'][] = $closedDays;
											$bases[$k]['custom_days'][] = $customDays;

											foreach ($bases[$k]['hours'] as $kk => $vv)
											{
												$bases[$k]['hours'][$kk] = array_merge($bases[$k]['hours'][$kk], $hours[$k]);
											}

											$bases[$k]['uid'][] = (string) $b->LocationDetail['Code'];
											$bases[$k]['uuid'][] = (string) $b->LocationDetail['ExtendedLocationCode'];

											$b = NULL;
										}
									}

									if ($b)
									{
										$bases[] = array
										(
											'uid' => array((string) $b->LocationDetail['Code']),
											'uuid' => array((string) $b->LocationDetail['ExtendedLocationCode']),
											'closed_days' => [$closedDays],
											'custom_days' => [$customDays],
											'hours' => $hours,
											'name' => $name
										);
									}
								}
							}

							usort($bases, function($lhs, $rhs) { return strcasecmp($lhs['name'], $rhs['name']); });
							for ($i = 0; $i < count($bases); $i++)
							{
								$bases[$i]['index'] = $i;
							}

							if (@count($result->VehClasses->VehClass) > 1)
							{
								foreach ($result->VehClasses->VehClass as $r)
								{
									$name = (string) $r['Name'];
									if (isset($this->STRINGS[$this->LANG]['booking_clazz_' . $name]))
									{
										$name = $this->STRINGS[$this->LANG]['booking_clazz_' . $name];
									}

									if ($name == '')
									{
										continue;
									}

									$codes = [];
									if (@count($r->VehType) > 1)
									{
										foreach ($r->VehType as $rr)
										{
											$code = (string) $rr['Code'];
											if ($code && $code != '')
											{
												$codes[] = $code;
											}
										}
									}
									else if ($r->VehType['Code'] && $r->VehType['Code'] != '')
									{
										$codes[] = (string) $r->VehType['Code'];
									}

									foreach ($classes as $k => $class)
									{
										if (strcasecmp($class['name'], $name) == 0)
										{
											$classes[$k]['codes'][] = $codes;
											$classes[$k]['uuid'][] = (string) $r['ExtendedClassCode'];

											$r = NULL;
										}
									}

									if ($r)
									{
										$classes[] = array
										(
											'codes' => array($codes),
											'uuid' => array((string) $r['ExtendedClassCode']),

											'icon' => (string) $r['IconURL'],
											'name' => (string) $name,
											'sorting_id' => (string) $r['Order'],
										);
									}
								}
							}

							// Sort the classes.
							usort($classes, function($lhs, $rhs) { return $lhs['sorting_id'] - $rhs['sorting_id']; });
						}
						else
						{
							$error = $this->printSoapError($client);
						}
					}

					if (isset($_SESSION[_SI]) && isset($_SESSION[_SI]['input']) && isset($_SESSION[_SI]['input']['ob'])
							&& isset($_SESSION[_SG]['bases'][$_SESSION[_SI]['input']['ob']]))
					{
						$res['checkout_base'] = $_SESSION[_SG]['bases'][$_SESSION[_SI]['input']['ob']];
					}

					if (isset($_SESSION[_SI]) && isset($_SESSION[_SI]['input']) && isset($_SESSION[_SI]['input']['ib'])
							&& isset($_SESSION[_SG]['bases'][$_SESSION[_SI]['input']['ib']]))
					{
						$res['checkin_base'] = $_SESSION[_SG]['bases'][$_SESSION[_SI]['input']['ib']];
					}

					if (isset($_SESSION[_SI]) && isset($_SESSION[_SI]['input']) && isset($_SESSION[_SI]['input']['ot']))
					{
						$res['checkout_time'] = $_SESSION[_SI]['input']['ot'];
					}

					if (isset($_SESSION[_SI]) && isset($_SESSION[_SI]['input']) && isset($_SESSION[_SI]['input']['it']))
					{
						$res['checkin_time'] = $_SESSION[_SI]['input']['it'];
					}

					if (isset($_SESSION[_SI]) && isset($_SESSION[_SI]['input']) && isset($_SESSION[_SI]['input']['yd']))
					{
						$res['young_driver'] = $_SESSION[_SI]['input']['yd'];
					}

					if (!$error && $bases)
					{
						if (count($bases) > 0)
						{
							$_SESSION[_SG]['bases'] = $bases;
							$_SESSION[_SG]['classes'] = $classes;
						}

						// Do we have to filter the classes?
						if (isset($this->PARAMS['form_classes_list']) && is_array($this->PARAMS['form_classes_list'])
								&& count($this->PARAMS['form_classes_list']) > 0)
						{
							$thisClasses = [];
							foreach ($this->PARAMS['form_classes_list'] as $k => $v)
							{
								foreach ($classes as $kk => $vv)
								{
									if (strtolower(trim($vv['name'])) == strtolower(trim($v)))
									{
										$thisClasses[] = $vv;
										break;
									}
								}
							}

							$classes = $thisClasses;
						}

						$res['base'] = $bases;
						$res['clazz'] = $classes;

						$res['success'] = TRUE;
					}
				}
				///
				///
				// CANCEL BOOKING
				///
				///
				else if ($params['cmd'] == 'cancel' && isset($params['reservation'])
						&& (($reservation = json_decode(stripcslashes($params['reservation']), TRUE)) != NULL
								|| ($reservation = json_decode($params['reservation'], TRUE)) != NULL))
				{
					$xml = '
						<ns1:OTA_VehModifyRQ>
						  <POS>
							<Source>
							  <RequestorID Type="29" ID="' . AUTH_ID . '" MessagePassword="' . htmlspecialchars(AUTH_API_KEY) . '"><CompanyName></CompanyName></RequestorID>
							</Source>
						  </POS>
						  <UniqueID Type="16" ID="' . $reservation['uid'] . '"/>
						  <VehModifyRQCore ModifyType="Cancel"/>
						</ns1:OTA_VehModifyRQ>';

					$client = new SoapClient($wsdl, array('trace' => 1, 'cache_wsdl' => WSDL_CACHE_NONE));
					$result = $client->OTA_VehModify(new SoapVar($xml, XSD_ANYXML));
					if ($result && isset($result->Success) && $result->Success)
					{
						$res['success'] = TRUE;

						// Cancel the payment.
						if (isset($params['payment']) && ($payment = json_decode(stripcslashes($params['payment']), TRUE)) != NULL)
						{
							if ($payment['status'] == 'auth' || $payment['status'] == 'confirmed')
							{
								$mippApiKey = get_option('otabo_mipp_api_key');
								if (!$mippApiKey || $mippApiKey == '')
								{
									$mippApiKey = get_option('otabo_mipp_api_key_' . $reservation['data']['vendor']['id']);
								}

								$osd = isset($params['osd']) ? $params['osd'] : NULL;

								$mipp = new MiPP($mippApiKey);
								$mipp->updatePayment($payment['uid'], ['operation' => 'invalidate']);
							}
						}
					}
					else
					{
						$error = $this->printSoapError($client);

// 						$cleanXml = str_ireplace(['SOAP-ENV:', 'SOAP:', 'ns1:'], '', $client->__getLastRequest());
// 						$xml = simplexml_load_string($cleanXml);
//
// 						$error .= "\n" . $xml->asXML();
//
// 						$cleanXml = str_ireplace(['SOAP-ENV:', 'SOAP:', 'ns1:'], '', $client->__getLastResponse());
// 						$xml = simplexml_load_string($cleanXml);
//
// 						$error .= "\n" . $xml->asXML();
					}
				}
				///
				///
				// CONFIRM BOOKING
				///
				///
				else if ($params['cmd'] == 'confirm' && isset($params['reservation']) && isset($params['payment'])
						&& (($reservation = json_decode(stripcslashes($params['reservation']), TRUE)) != NULL
						&& ($payment = json_decode(stripcslashes($params['payment']), TRUE)) != NULL
						|| ($reservation = json_decode($params['reservation'], TRUE)) != NULL
						&& ($payment = json_decode($params['payment'], TRUE)) != NULL))
				{
					$xml = '
						<ns1:OTA_VehModifyRQ>
						  <POS>
							<Source>
							  <RequestorID Type="29" ID="' . AUTH_ID . '" MessagePassword="' . htmlspecialchars(AUTH_API_KEY) . '"><CompanyName></CompanyName></RequestorID>
							</Source>
						  </POS>
						  <UniqueID Type="16" ID="' . $reservation['uid'] . '"/>
						  <VehModifyRQCore ModifyType="Book"/>
						  <VehModifyRQInfo>
						    <RentalPaymentPref>
						      <Voucher Identifier="' . $payment['identifier'] . '"/>
						      <PaymentAmount CurrencyCode="EUR" Amount="' . sprintf('%.02f', $payment['amount']) . '"/>
						    </RentalPaymentPref>
						  </VehModifyRQInfo>
						</ns1:OTA_VehModifyRQ>';

					$client = new SoapClient($wsdl, array('trace' => 1, 'cache_wsdl' => WSDL_CACHE_NONE));
					$result = $client->OTA_VehModify(new SoapVar($xml, XSD_ANYXML));
					if ($result && isset($result->Success) && $result->Success)
					{
						$cleanXml = str_ireplace(['SOAP-ENV:', 'SOAP:', 'ns1:'], '', $client->__getLastResponse());
						$xml = simplexml_load_string($cleanXml);
						$xml = $xml->Body->OTA_VehModifyRS->VehModifyRSCore->VehReservation;

						$res['success'] = TRUE;

						$customer = $xml->Customer->Primary;
						$res['client'] = array
						(
							'name' => $customer->PersonName && $customer->PersonName->GivenName ? (string) $customer->PersonName->GivenName : '',
							'surname' => $customer->PersonName && $customer->PersonName->Surname ? (string) $customer->PersonName->Surname : '',
							'email' => $customer->Email ? (string) $customer->Email : '',
							'email_confirm' => $customer->Email ? (string) $customer->Email : '',
							'phone' => $customer->Telephone && $customer->Telephone['PhoneNumber'] ? (string) $customer->Telephone['PhoneNumber'] : '',
							'address' => $customer->Address && $customer->Address->AddressLine ? (string) $customer->Address->AddressLine : '',
							'city' => $customer->Address && $customer->Address->CityName ? (string) $customer->Address->CityName : '',
							'country' => $customer->Address && $customer->Address->CountryName ? (string) $customer->Address->CountryName : '',
							'zip_code' => $customer->Address && $customer->Address->PostalCode ? (string) $customer->Address->PostalCode : '',
							'province' => $customer->Address && $customer->Address->StateProv ? (string) $customer->Address->StateProv : '',
						);

						$res['reservation'] = array
						(
							'status' => $xml['ReservationStatus'] ? (string) $xml['ReservationStatus'] : NULL,
							'totals' => $xml->VehSegmentCore->TotalCharge && $xml->VehSegmentCore->TotalCharge['EstimatedTotalAmount'] ?
									(string) $xml->VehSegmentCore->TotalCharge['EstimatedTotalAmount'] : NULL
						);

						$agentId = NULL;
						foreach ($xml->VehSegmentCore->ConfID as $k => $v)
						{
							if ($v['Type'] && (string) $v['Type'] == '16')
							{
								$res['reservation']['uid'] = (string) $v['ID'];
							}
							else if ($v['Type'] && (string) $v['Type'] == '14')
							{
								$res['reservation']['ref_id'] = (string) $v['ID'];
							}
							else if ($v['Type'] && (string) $v['Type'] == '29')
							{
								$agentId = (string) $v['ID'];
								$res['reservation']['data'] = ['vendor' => ['id' => $agentId]];
							}
						}
					}
					else
					{
						$error = $this->printSoapError($client);

						// Cancel the payment.
						if ($payment['status'] == 'auth' || $payment['status'] == 'confirmed')
						{
							$mippApiKey = get_option('otabo_mipp_api_key');
							if (!$mippApiKey || $mippApiKey == '')
							{
								$mippApiKey = get_option('otabo_mipp_api_key_' . $reservation['data']['vendor']['id']);
							}

							$osd = isset($params['osd']) ? $params['osd'] : NULL;

							$mipp = new MiPP($mippApiKey);
							$mipp->updatePayment($payment['uid'], ['operation' => 'invalidate']);
						}

						// Cancel the reservation.
						$xml = '
							<ns1:OTA_VehModifyRQ>
							  <POS>
								<Source>
								  <RequestorID Type="29" ID="' . AUTH_ID . '" MessagePassword="' . htmlspecialchars(AUTH_API_KEY) . '"><CompanyName></CompanyName></RequestorID>
								</Source>
							  </POS>
							  <UniqueID Type="16" ID="' . $reservation['uid'] . '"/>
							  <VehModifyRQCore ModifyType="Cancel"/>
							</ns1:OTA_VehModifyRQ>';

						$client = new SoapClient($wsdl, array('trace' => 1, 'cache_wsdl' => WSDL_CACHE_NONE));
						$result = $client->OTA_VehModify(new SoapVar($xml, XSD_ANYXML));
						if ($result && isset($result->Success) && $result->Success)
						{
						}
//
// 						$cleanXml = str_ireplace(['SOAP-ENV:', 'SOAP:', 'ns1:'], '', $client->__getLastRequest());
// 						$xml = simplexml_load_string($cleanXml);
//
// 						$error .= "\n" . $xml->asXML();
//
// 						$cleanXml = str_ireplace(['SOAP-ENV:', 'SOAP:', 'ns1:'], '', $client->__getLastResponse());
// 						$xml = simplexml_load_string($cleanXml);
//
// 						$error .= "\n" . $xml->asXML();
					}
				}
				///
				///
				// RETRIEVAL
				///
				///
				else if ($params['cmd'] == 'load' && isset($params['rid']))
				{
					$xml = '
						<ns1:OTA_VehRetResRQ>
						  <POS>
							<Source>
							  <RequestorID Type="29" ID="' . AUTH_ID . '" MessagePassword="' . htmlspecialchars(AUTH_API_KEY) . '"><CompanyName></CompanyName></RequestorID>
							</Source>
						  </POS>
						  <VehRetResRQCore>
						    <UniqueID Type="16" ID="' . $params['rid'] . '"/>
						  </VehRetResRQCore>
						</ns1:OTA_VehRetResRQ>';

					$client = new SoapClient($wsdl, array('trace' => 1, 'cache_wsdl' => WSDL_CACHE_NONE));
					$result = $client->OTA_VehRetRes(new SoapVar($xml, XSD_ANYXML));
					if ($result && isset($result->Success) && $result->Success)
					{
						$cleanXml = str_ireplace(['SOAP-ENV:', 'SOAP:', 'ns1:'], '', $client->__getLastResponse());
						$xml = simplexml_load_string($cleanXml);
						$xml = $xml->Body->OTA_VehRetResRS->VehRetResRSCore->VehReservation;

						$res['success'] = TRUE;
						$res['reservation'] = array
						(
							'status' => $xml['ReservationStatus'] ? (string) $xml['ReservationStatus'] : NULL,
							'totals' => $xml->VehSegmentCore->TotalCharge && $xml->VehSegmentCore->TotalCharge['EstimatedTotalAmount'] ?
									(string) $xml->VehSegmentCore->TotalCharge['EstimatedTotalAmount'] : NULL
						);

						$perc = $xml->VehSegmentCore && $xml->VehSegmentCore->Vendor && $xml->VehSegmentCore->Vendor['DPay'] ?
								(string) $xml->VehSegmentCore->Vendor['DPay'] : 1;
						$toPay = $res['reservation']['totals'];

						if (is_numeric($perc) && $perc > 0)
						{
							$toPay = sprintf('%.02f', $toPay * $perc);
						}

						$res['reservation']['totals_to_pay'] = $toPay;

						$agentId = NULL;
						foreach ($xml->VehSegmentCore->ConfID as $k => $v)
						{
							if ($v['Type'] && (string) $v['Type'] == '16')
							{
								$res['reservation']['uid'] = (string) $v['ID'];
							}
							else if ($v['Type'] && (string) $v['Type'] == '14')
							{
								$res['reservation']['ref_id'] = (string) $v['ID'];
							}
							else if ($v['Type'] && (string) $v['Type'] == '29')
							{
								$agentId = (string) $v['ID'];
								$res['reservation']['data'] = ['vendor' => ['id' => $agentId]];
							}
						}

						// Should we load payment information as well?
						if (isset($params['mpi']))
						{
							require_once(__DIR__ . '/mipp.class.php');

							// Initiate the payment.
							$mippApiKey = get_option('otabo_mipp_api_key');
							if ((!$mippApiKey || $mippApiKey == '') && $agentId)
							{
								$mippApiKey = get_option('otabo_mipp_api_key_' . $agentId);
							}

							$mipp = new MiPP($mippApiKey);
							$payRes = $mipp->getPayment($params['mpi']);
							if (isset($payRes['success']))
							{
								$payment = $payRes['result']['payment'];
								if (isset($payment['params']) && (is_array($payment['params']) || ($payment['params'] = json_decode($payment['params'], TRUE)) != NULL)
										&& isset($payment['params']['metadata']) && isset($payment['params']['metadata']['rid'])
										&& $payment['params']['metadata']['rid'] == $res['reservation']['uid'])
								{
									$res['payment'] = $payment;
								}
							}
							else
							{
								$error = $payRes['error'];
								$res['success'] = FALSE;
							}
						}
					}
					else
					{
						$error = $this->printSoapError($client);

// 						$cleanXml = str_ireplace(['SOAP-ENV:', 'SOAP:', 'ns1:'], '', $client->__getLastRequest());
// 						$xml = simplexml_load_string($cleanXml);
//
// 						$error .= "\nRequest: " . $xml->asXML();
//
// 						$cleanXml = str_ireplace(['SOAP-ENV:', 'SOAP:', 'ns1:'], '', $client->__getLastResponse());
// 						$xml = simplexml_load_string($cleanXml);
//
// 						$error .= "\nResponse: " . $xml->asXML();
					}
				}
				///
				///
				// RESULTS
				///
				///
				else if ($params['cmd'] == 'results'
						&& (isset($params['ob']) && isset($params['ot']) && isset($params['it'])
						|| isset($_SESSION[_SI]) && isset($_SESSION[_SI]['input']) && isset($_SESSION[_SI]['input']['ob'])
								&& isset($_SESSION[_SI]['input']['ot']) && isset($_SESSION[_SI]['input']['it'])))
				{
					if (!isset($_SESSION[_SG]) || !isset($_SESSION[_SG]['bases']))
					{
						$error = 'Error 24001: ' . _SG;
					}
					else
					{
						$bases = $_SESSION[_SG]['bases'];

						if (isset($_SESSION[_SI]['quotes_result']) && count($_SESSION[_SI]['quotes_result']) > 0
								&& (!isset($params['ob']) || $_SESSION[_SI]['input']['ob'] == $params['ob'])
								&& (!isset($params['ib']) || $_SESSION[_SI]['input']['ib'] == $params['ib'])
								&& (!isset($params['ot']) || $_SESSION[_SI]['input']['ot'] == $params['ot'])
								&& (!isset($params['it']) || $_SESSION[_SI]['input']['it'] == $params['it'])
								&& (!isset($params['oc']) || $_SESSION[_SI]['input']['oc'] == $params['oc']))
						{
							$res['result'] = $_SESSION[_SI]['quotes_result'];
							$res['success'] = TRUE;
						}

						if (isset($params['ob']))
						{
							$_SESSION[_SI] =
							[
								'input' =>
								[
									'ob' => isset($params['ob']) ? $params['ob'] : NULL,
									'ib' => isset($params['ib']) ? $params['ib'] : NULL,
									'ot' => isset($params['ot']) ? $params['ot'] : NULL,
									'it' => isset($params['it']) ? $params['it'] : NULL,
									'oc' => isset($params['oc']) ? $params['oc'] : NULL,
									'yd' => isset($params['yd']) ? $params['yd'] : NULL,
								]
							];
						}

						// Get the bases.
						$checkoutBase = isset($bases[$_SESSION[_SI]['input']['ob']]) ? $bases[$_SESSION[_SI]['input']['ob']] : NULL;
						$checkinBase = isset($_SESSION[_SI]['input']['ib']) && strlen($_SESSION[_SI]['input']['ib']) > 0 ? (isset($bases[$_SESSION[_SI]['input']['ib']]) ? $bases[$_SESSION[_SI]['input']['ib']] : NULL) :
								(isset($bases[$_SESSION[_SI]['input']['ob']]) ? $bases[$_SESSION[_SI]['input']['ob']] : NULL);
						if ($checkoutBase && $checkinBase && !isset($res['result']))
						{
							$input = array
							(
								'checkout_base' => $checkoutBase,
								'checkout_time' => $_SESSION[_SI]['input']['ot'],
								'checkin_base' => $checkinBase,
								'checkin_time' => $_SESSION[_SI]['input']['it'],
							);

							// Only consider companies that serve both bases.
							$checkoutBases = array();
							$checkinBases = array();

							for ($i = count($checkoutBase['uuid']) - 1; $i >= 0; $i--)
							{
								$uuid = $checkoutBase['uuid'][$i];
								$index = array_search($uuid, $checkinBase['uuid']);

								if ($index !== FALSE)
								{
									$checkoutBases[] = $checkoutBase['uid'][$i];
									$checkinBases[] = $checkinBase['uid'][$index];
								}
							}

							$classes = '';
							if (isset($_SESSION[_SI]['input']['oc']) && isset($_SESSION[_SG]['classes']) && isset($_SESSION[_SG]['classes'][$_SESSION[_SI]['input']['oc']]))
							{
								$class = $_SESSION[_SG]['classes'][$_SESSION[_SI]['input']['oc']];
								foreach ($class['codes'] as $k => $v)
								{
									foreach ($v as $kk => $vv)
									{
										$classes .= '<VehPref><VehMakeModel Code="' . htmlspecialchars($vv) . '" '
												. 'ExtendedVehicleCode="' . htmlspecialchars($class['uuid'][$k]) . '" /></VehPref>';
									}
								}
							}

							$xml = '
								<ns1:OTA_VehAvailRateRQ>
								  <POS>
									<Source>
									  <RequestorID Type="29" ID="' . AUTH_ID . '" MessagePassword="' . htmlspecialchars(AUTH_API_KEY) . '"><CompanyName></CompanyName></RequestorID>
									</Source>
								  </POS>
								  <VehAvailRQCore>
									<VehRentalCore PickUpDateTime="' . str_replace(' ', 'T', $_SESSION[_SI]['input']['ot']) . 'Z" '
											. 'ReturnDateTime="' . str_replace(' ', 'T', $_SESSION[_SI]['input']['it']) . 'Z">
									  <PickUpLocation LocationCode="' . implode(',', $checkoutBases) . '"/>
									  <ReturnLocation LocationCode="' . implode(',', $checkinBases) . '"/>
									</VehRentalCore>
									' . ($classes != '' ? '<VehPrefs>' . $classes . '</VehPrefs>' : '') . '
								  </VehAvailRQCore>
								</ns1:OTA_VehAvailRateRQ>';

							$client = new SoapClient($wsdl, array('trace' => 1, 'cache_wsdl' => WSDL_CACHE_NONE));
							$result = $client->OTA_VehAvailRate(new SoapVar($xml, XSD_ANYXML));

							$quotes = array();
							$groups = array();

							if ($result && isset($result->Success) && $result->Success)
							{
								$cleanXml = str_ireplace(['SOAP-ENV:', 'SOAP:', 'ns1:'], '', $client->__getLastResponse());
								$xml = simplexml_load_string($cleanXml, NULL, LIBXML_NOBLANKS);
								$result = $xml->Body->OTA_VehAvailRateRS->VehAvailRSCore->VehVendorAvails;

								foreach ($result->VehVendorAvail as $vendorResult)
								{
									$vendor = NULL;
									if (isset($vendorResult->Vendor))
									{
										$vendor = array
										(
											'dpay' => $vendorResult->Vendor['DPay'] && is_numeric((string) $vendorResult->Vendor['DPay']) ? (string) $vendorResult->Vendor['DPay'] : 1,
											'id' => (string) $vendorResult->Vendor['Code'],
											'name' => (string) $vendorResult->Vendor['CompanyShortName'],
											'logo' => (string) $vendorResult->Vendor['Logo'],
											'privacy_url' => (string) $vendorResult->Vendor->PrivacyDocument,
											'terms_url' => (string) $vendorResult->Vendor->TermsDocument
										);
									}

									if (!$vendorResult->VehAvails)
									{
										continue;
									}

									foreach ($vendorResult->VehAvails->VehAvail as $avail)
									{
										$charges = '';
										$optionals = [];

										$rateVatExcl = 0;
										$rateVatIncl = 0;
										$totals = 0;

										foreach ($avail->VehAvailCore->RentalRate->VehicleCharges->VehicleCharge as $charge)
										{
											$purpose = (string) $charge['Purpose'];
											if ($purpose == '21')
											{
												$thisRate = (string) $charge['Amount'];
												if ($charge->TaxAmounts)
												{
													foreach ($charge->TaxAmounts->TaxAmount as $tax)
													{
														$thisRate += (string) $tax['Total'];
													}
												}

												$totals += $thisRate;
											}
											else if ($purpose != '1')
											{
												$charges .= $charge->asXML();
												$thisRate = (string) $charge['Amount'];
												if ($charge->TaxAmounts)
												{
													foreach ($charge->TaxAmounts->TaxAmount as $tax)
													{
														$thisRate += (string) $tax['Total'];
													}
												}

												$name = (string) $charge['Description'];
												if (isset($this->STRINGS[$this->LANG]['booking_charge_' . $purpose . '_title']))
												{
													$name = $this->STRINGS[$this->LANG]['booking_charge_' . $purpose . '_title'];
												}

												$desc = NULL;
												if (isset($this->STRINGS[$this->LANG]['booking_charge_' . $purpose . '_desc']))
												{
													$desc = $this->STRINGS[$this->LANG]['booking_charge_' . $purpose . '_desc'];
												}

												$optionals[] = array
												(
													'countable' => FALSE,
													'desc' => $desc,
													'name' => $name,
													'rate' => $thisRate,
													'required' => TRUE,
												);

												$totals += $thisRate;
											}
											else
											{
												$charges .= $charge->asXML();
												$rateVatExcl = (string) $charge['Amount'];
												$rateVatIncl = $rateVatExcl;

												if ($charge->TaxAmounts)
												{
													foreach ($charge->TaxAmounts->TaxAmount as $tax)
													{
														$rateVatIncl += (string) $tax['Total'];
													}
												}

												$totals += $rateVatIncl;
											}
										}

										$vehicleParams = [];
										if ($avail->VehAvailCore->Vehicle->VehType)
										{
											foreach ($avail->VehAvailCore->Vehicle->VehType->children() as $kk => $vv)
											{
												$vehicleParams[$kk] = array();
												foreach ($vv as $kkk => $vvv)
												{
													$kkk = (string) $kkk;
													if ($vvv->attributes())
													{
														$vehicleParams[$kk][$kkk] = [];
														foreach ($vvv->attributes() as $kkkk => $vvvv)
														{
															$vehicleParams[$kk][$kkk][$kkkk] = (string) $vvvv == 'true';
														}
													}
													else
													{
														$vehicleParams[$kk][$kkk] = nl2br((string) $vvv);
													}
												}
											}
										}

										$group = $avail->VehAvailCore->Vehicle->VehType ?
												(string) $avail->VehAvailCore->Vehicle->VehType['VehicleCategory'] :
												(string) $avail->VehAvailCore->Vehicle['Description'];
										if (isset($this->STRINGS[$this->LANG]['booking_group_' . $group]))
										{
											$group = $this->STRINGS[$this->LANG]['booking_group_' . $group];
										}

										$name = (string) $avail->VehAvailCore->Vehicle['Description'];
										if (isset($this->STRINGS[$this->LANG]['booking_vehicle_' . $name]))
										{
											$name = $this->STRINGS[$this->LANG]['booking_vehicle_' . $name];
										}

										$tariff = (string) $avail->VehAvailCore->Vehicle->Tariff;
										if (isset($this->STRINGS[$this->LANG]['booking_tariff_' . $tariff]))
										{
											$tariff = $this->STRINGS[$this->LANG]['booking_tariff_' . $tariff];
										}

										$quote = array
										(
											'acriss_code' => (string) $avail->VehAvailCore->Vehicle['Code'],
											'group' => $group,
											'image' => (string) $avail->VehAvailCore->Vehicle->PictureURL,
											'name' => $name,
											'params' => $vehicleParams,
											'rate' => $rateVatIncl,
											'rate_vat_excl' => $rateVatExcl,
											'tariff' => $tariff,
											'charges' => implode('', array_map('trim', explode("\n", $charges))),
											'vendor' => $vendor,
											'coverages' => array(),
											'optionals' => $optionals,
										);

										if ($avail->VehAvailCore->RentalRate
												&& $avail->VehAvailCore->RentalRate->RateDistance
												&& $avail->VehAvailCore->RentalRate->RateDistance['Unlimited'] == 'false')
										{
											$quote['km_limit'] = (string) $avail->VehAvailCore->RentalRate->RateDistance['Quantity'];
											$quote['km_limit_period'] = $strings['results_km_limit_' . strtolower((string) $avail->VehAvailCore->RentalRate->RateDistance['VehiclePeriodUnitName'])];
										}
										else if ($avail->VehAvailCore->RentalRate && $avail->VehAvailCore->RentalRate->RateDistance)
										{
											$quote['km_limit'] = FALSE;
											$quote['km_limit_period'] = $strings['results_km_limit_' . strtolower((string) $avail->VehAvailCore->RentalRate->RateDistance['VehiclePeriodUnitName'])];
										}

										if ($avail->VehAvailCore->PricedEquips)
										{
											foreach ($avail->VehAvailCore->PricedEquips->PricedEquip as $optional)
											{
												if ($optional->Equipment && $optional->Equipment['EquipType'] && (string) $optional->Equipment['EquipType'] != '39')
												{
													if (!$optional->Charge)
													{
														continue;
													}

													$rate = (string) $optional->Charge['Amount'];
													if ($optional->Charge->TaxAmounts)
													{
														foreach ($optional->Charge->TaxAmounts->TaxAmount as $kk => $tax)
														{
															$rate += (string) $tax['Total'];
														}
													}

													$charges = '<VehicleCharge Purpose="21" '
															. 'Description="' . htmlspecialchars((string) $optional->Equipment->Description) . '" '
															. 'IncludedInEstTotalInd="true" '
															. 'IncludedInRate="false" '
															. 'TaxInclusive="false" '
															. 'Amount="' . (string) $optional->Charge['Amount'] . '" '
															. 'CurrencyCode="EUR">' . $optional->Charge->TaxAmounts->asXML() . '</VehicleCharge>';

													$name = (string) $optional->Equipment->Description;
													if (isset($this->STRINGS[$this->LANG]['booking_optionals_' . (string) $optional->Equipment['EquipType'] . '_title']))
													{
														$name = $this->STRINGS[$this->LANG]['booking_optionals_' . (string) $optional->Equipment['EquipType'] . '_title'];
													}

													$desc = NULL;
													if (isset($this->STRINGS[$this->LANG]['booking_optionals_' . (string) $optional->Equipment['EquipType'] . '_desc']))
													{
														$desc = $this->STRINGS[$this->LANG]['booking_optionals_' . (string) $optional->Equipment['EquipType'] . '_desc'];
													}

													$quote['optionals'][] = array
													(
														'countable' => $optional->Equipment['Quantity'] && $optional->Equipment['Quantity'] != 0,
														'id' => (string) $optional->Equipment['EquipType'],
														'desc' => $desc,
														'name' => $name,
														'rate' => $rate,
														'charges' => implode('', array_map('trim', explode("\n", $charges))),
														'required' => $optional->Charge['IncludedInRate'] && strtolower($optional->Charge['IncludedInRate']) == 'true'
																|| $optional->Charge['IncludedInEstTotalInd'] && strtolower($optional->Charge['IncludedInEstTotalInd']) == 'true',
													);
												}
											}
										}

										if ($avail->VehAvailInfo && $avail->VehAvailInfo->PricedCoverages)
										{
											foreach ($avail->VehAvailInfo->PricedCoverages->PricedCoverage as $coverage)
											{
												if ($coverage->Coverage && $coverage->Coverage['CoverageType'] && (string) $coverage->Coverage['CoverageType'] == '21')
												{
													if (!$coverage->Charge)
													{
														continue;
													}

													$rate = (string) $coverage->Charge['Amount'];
													if ($coverage->Charge->TaxAmounts)
													{
														foreach ($coverage->Charge->TaxAmounts->TaxAmount as $tax)
														{
															$rate += (string) $tax['Total'];
														}
													}

													$charges = '<VehicleCharge Purpose="4" '
															. 'Description="' . htmlspecialchars((string) $coverage->Coverage->Details['CoverageTextType']) . '" '
															. 'IncludedInEstTotalInd="true" '
															. 'IncludedInRate="false" '
															. 'TaxInclusive="false" '
															. 'Amount="' . htmlspecialchars((string) $coverage->Charge['Amount']) . '" '
															. 'CurrencyCode="EUR">' . $coverage->Charge->TaxAmounts->asXML() . '</VehicleCharge>';

													$name = (string) $coverage->Coverage->Details['CoverageTextType'];
													if (isset($this->STRINGS[$this->LANG]['booking_coverage_' . (string) $coverage->Coverage['Code'] . '_title']))
													{
														$name = $this->STRINGS[$this->LANG]['booking_coverage_' . (string) $coverage->Coverage['Code'] . '_title'];
													}

													$desc = NULL;
													if (isset($this->STRINGS[$this->LANG]['booking_coverage_' . (string) $coverage->Coverage['Code'] . '_desc']))
													{
														$desc = $this->STRINGS[$this->LANG]['booking_coverage_' . (string) $coverage->Coverage['Code'] . '_desc'];
													}

													$quote['coverages'][] = array
													(
														'id' => (string) $coverage->Coverage['Code'],
														'name' => $name,
														'desc' => $desc,
														'rate' => $rate,
														'charges' => implode('', array_map('trim', explode("\n", $charges))),
														'deposit' => (string) $coverage->Coverage['Deposit'],
														'excess' => isset($coverage->Deductible['Amount']) ? (string) $coverage->Deductible['Amoun'] : NULL,
														'excess_damage' => isset($coverage->Deductible['ExcessDamage']) ? (string) $coverage->Deductible['ExcessDamage'] : NULL,
														'excess_info' => isset($coverage->Deductible['ExcessInfo']) ? (string) $coverage->Deductible['ExcessInfo'] : NULL,
														'excess_theft' => isset($coverage->Deductible['ExcessTheft']) ? (string) $coverage->Deductible['ExcessTheft'] : NULL,
													);
												}
											}

											usort($quote['coverages'], function($lhs, $rhs) { return $lhs['rate'] - $rhs['rate']; });
										}

										if ($this->PARAMS['result_full_width'])
										{
											if (isset($quotes[$quote['acriss_code']]))
											{
												$quotes[$quote['acriss_code']]['options'][] = $quote;
											}
											else
											{
												$quote['options'] = [];
												$quotes[$quote['acriss_code']] = $quote;
											}
										}
										else
										{
											$quotes[] = $quote;
										}

										if (!isset($groups[$quote['group']]))
										{
											$groups[$quote['group']] = array
											(
												'min' => $quote['rate'],
												'name' => $quote['group']
											);
										}

										if (!isset($groups[$quote['group']]['image']) && isset($quote['image']) && strlen($quote['image']) > 0)
										{
											$groups[$quote['group']]['image'] = $quote['image'];
										}

										if ($groups[$quote['group']]['min'] > $quote['rate'])
										{
											$groups[$quote['group']]['min'] = $quote['rate'];
										}
									}
								}

								$thisGroups = array();
								foreach ($groups as $k => $v)
								{
									$v['name'] = $k;
									$thisGroups[] = $v;
								}

								$groups = $thisGroups;
								usort($groups, function($lhs, $rhs)
								{
									$res = (isset($lhs['min']) ? $lhs['min'] : 0) - (isset($rhs['min']) ? $rhs['min'] : 0);
									if ($res == 0)
									{
										$res = strcasecmp($lhs['name'], $rhs['name']);
									}

									return $res;
								});

								usort($quotes, function($lhs, $rhs)
								{
									return $lhs['rate'] == $rhs['rate'] ?
											strcasecmp($lhs['group'], $rhs['group']) :
											$lhs['rate'] - $rhs['rate'];
								});

								foreach ($quotes as $k => $v)
								{
									$quotes[$k]['id'] = $k;
								}

								$res['result'] = array('quotes' => $quotes, 'groups' => $groups, 'input' => $input);
								$res['success'] = TRUE;
							}
							else
							{
								$cleanXml = str_ireplace(['SOAP-ENV:', 'SOAP:', 'ns1:'], '', $client->__getLastResponse());
								$xml = simplexml_load_string($cleanXml);

								$error = 'Error 24002 - ';
								$error .= isset($xml->Body->OTA_VehAvailRateRS->Errors->Error[0]['ShortText']) ?
										$xml->Body->OTA_VehAvailRateRS->Errors->Error[0]['ShortText'] :
										(string) $xml->Body->OTA_VehAvailRateRS->Errors->Error[0];
// 								$error .= $cleanXml;
							}
						}
						else if (!isset($res['result']))
						{
							$error = 'Error 24003 - ' . $params['ob'] . ' :: ' . (isset($params['ib']) ? $params['ib'] : 'NA');
						}
					}

					// Apply or remove the young driver optional?
					if (isset($res['success']) && $res['success'])
					{
						foreach ($res['result']['quotes'] as $k => $v)
						{
							// Check whether we've got a young driver extra and mark it as mandatory.
							if (!isset($v['optionals']))
							{
								continue;
							}

							foreach ($v['optionals'] as $kk => $vv)
							{
								if (isset($_SESSION[_SI]['input']['yd']) && $_SESSION[_SI]['input']['yd'] != 0 && isset($vv['id']) && $vv['id'] == 102
										&& (!isset($vv['required']) || !$vv['required']))
								{
									$res['result']['quotes'][$k]['optionals'][$kk]['custom_require'] = TRUE;
									$res['result']['quotes'][$k]['optionals'][$kk]['required'] = TRUE;
								}
								else if (!(isset($_SESSION[_SI]['input']['yd']) && $_SESSION[_SI]['input']['yd'] != 0) && isset($vv['id']) && $vv['id'] == 102
										&& isset($vv['required']) && $vv['required']
										&& isset($vv['custom_require']) && $vv['custom_require'])
								{
									unset($res['result']['quotes'][$k]['optionals'][$kk]['custom_require']);
									$res['result']['quotes'][$k]['optionals'][$kk]['required'] = FALSE;
								}
							}

							if (isset($v['options']))
							{
								foreach ($v['options'] as $kkk => $vvv)
								{
									if (!isset($vvv['optionals']))
									{
										continue;
									}

									foreach ($vvv['optionals'] as $kk => $vv)
									{
										if (isset($_SESSION[_SI]['input']['yd']) && $_SESSION[_SI]['input']['yd'] != 0 && isset($vv['id']) && $vv['id'] == 102
												&& (!isset($vv['required']) || !$vv['required']))
										{
											$res['result']['quotes'][$k]['options'][$kkk]['optionals'][$kk]['custom_require'] = TRUE;
											$res['result']['quotes'][$k]['options'][$kkk]['optionals'][$kk]['required'] = TRUE;
										}
										else if (!(isset($_SESSION[_SI]['input']['yd']) && $_SESSION[_SI]['input']['yd'] != 0) && isset($vv['id']) && $vv['id'] == 102
												&& isset($vv['required']) && $vv['required']
												&& isset($vv['custom_require']) && $vv['custom_require'])
										{
											unset($res['result']['quotes'][$k]['options'][$kkk]['optionals'][$kk]['custom_require']);
											$res['result']['quotes'][$k]['options'][$kkk]['optionals'][$kk]['required'] = FALSE;
										}
									}
								}
							}
						}

						$_SESSION[_SI]['quotes_result'] = $res['result'];
					}
				}
				///
				///
				// PAYMENT
				///
				///
				else if ($params['cmd'] == 'payment' && isset($params['reservation'])
						&& (($reservation = json_decode(stripcslashes($params['reservation']), TRUE)) != NULL
						|| ($reservation = json_decode($params['reservation'], TRUE)) != NULL))
				{
					require_once(__DIR__ . '/mipp.class.php');

					// Initiate the payment.
					$mippApiKey = get_option('otabo_mipp_api_key');
					if (!$mippApiKey || $mippApiKey == '')
					{
						$mippApiKey = get_option('otabo_mipp_api_key_' . $reservation['data']['vendor']['id']);
					}

					$osd = isset($params['osd']) ? $params['osd'] : NULL;

					$mipp = new MiPP($mippApiKey);
					$res = $mipp->postPayment(
					[
						'amount' => sprintf('%.02f', $reservation['totals_to_pay']),
						'description' => 'Prenotazione ' . $reservation['ref_id'],
						'callback' =>
						[
							'url' => $this->getI18nPageLink(get_option('otabo_booking_url')) . '?' . ($osd ? '&osd=' . $osd . '&' : '') . 'rid=' . $reservation['uid']
						],
						'metadata' =>
						[
							'customer' => ['name' => $reservation['client_name']],
							'rid' => $reservation['uid']
						],
						'off_session' => TRUE,
						'operation' => 'auth',
					]);
				}
				///
				///
				// SUBMISSION
				///
				///
				else if ($params['cmd'] == 'submit' && (isset($params['quote'])
						&& (($quote = json_decode(stripcslashes($params['quote']), TRUE)) != NULL
						|| ($quote = json_decode($params['quote'], TRUE)) != NULL))
						&& isset($_SESSION[_SI]) && isset($_SESSION[_SI]['input']) && isset($_SESSION[_SI]['input']['ob'])
						&& isset($_SESSION[_SI]['input']['ot']) && isset($_SESSION[_SI]['input']['it']))
				{
					if (!isset($_SESSION[_SG]) || !isset($_SESSION[_SG]['bases']))
					{
						$error = 'Error 25001: ' . _SG;
					}
					else
					{
						// Get the bases.
						$bases = $_SESSION[_SG]['bases'];
						$checkoutBase = isset($bases[$_SESSION[_SI]['input']['ob']]) ? $bases[$_SESSION[_SI]['input']['ob']] : NULL;
						$checkinBase = isset($_SESSION[_SI]['input']['ib']) && strlen($_SESSION[_SI]['input']['ib']) > 0 ? (isset($bases[$_SESSION[_SI]['input']['ib']]) ? $bases[$_SESSION[_SI]['input']['ib']] : NULL) :
								(isset($bases[$_SESSION[_SI]['input']['ob']]) ? $bases[$_SESSION[_SI]['input']['ob']] : NULL);

						if (!$error && $checkoutBase && $checkinBase && !isset($res['result']))
						{
							$mippApiKey = get_option('otabo_mipp_api_key');
							if (!$mippApiKey || $mippApiKey == '')
							{
								$mippApiKey = get_option('otabo_mipp_api_key_' . $quote['vendor']['id']);
							}

							$input = array
							(
								'checkout_base' => $checkoutBase,
								'checkout_time' => $_SESSION[_SI]['input']['ot'],
								'checkin_base' => $checkinBase,
								'checkin_time' => $_SESSION[_SI]['input']['it'],
							);

							// Get the base ids for the vendor.
							$checkoutBases = array();
							$checkinBases = array();

							for ($i = count($checkoutBase['uuid']) - 1; $i >= 0; $i--)
							{
								$uuid = $checkoutBase['uuid'][$i];
								if ($uuid == (isset($quote['vendor']) ? $quote['vendor']['id'] : AUTH_ID))
								{
									$checkoutBases[] = $checkoutBase['uid'][$i];
									break;
								}
							}

							if ($checkinBase)
							{
								for ($i = count($checkinBase['uuid']) - 1; $i >= 0; $i--)
								{
									$uuid = $checkinBase['uuid'][$i];
									if ($uuid == (isset($quote['vendor']) ? $quote['vendor']['id'] : AUTH_ID))
									{
										$checkinBases[] = $checkinBase['uid'][$i];
										break;
									}
								}
							}
							else
							{
								$checkinBases = $checkoutBases;
							}

							$totals = $quote['rate'];
							if (isset($quote['coverage']))
							{
								$totals += $quote['coverage']['rate'];
							}

							if (isset($quote['optionals']))
							{
								$totals += array_reduce($quote['optionals'], function($s, $e) { return $s + $e['rate'] * (isset($e['quantity']) ? $e['quantity'] : 1); }, 0);
							}

							// Invoicing data?
							$invoicing = '';
							if (isset($quote['client']['invoicing_name']) && $quote['client']['invoicing_name'] != ''
									|| isset($quote['client']['invoicing_company']) && $quote['client']['invoicing_company'] != '')
							{
								$c = $quote['client'];
								$invoicing = '<Invoicing Type="' . ($c['invoicing_type'] == 'business' ? 'Business' : ($c['invoicing_type'] == 'one_man_business' ? 'Individual Company' : ($c['invoicing_type'] == 'public_amm' ? 'PA' : 'Private'))) . '">'
										. ($c['invoicing_type'] != 'private' && isset($c['invoicing_company']) && $c['invoicing_company'] != '' ? '<Name>' . htmlspecialchars($c['invoicing_company']) . '</Name>' : '')
										. ($c['invoicing_type'] == 'private' ?
												'<PersonName>'
														. (isset($c['invoicing_name']) && $c['invoicing_name'] != '' ? '<GivenName>' . htmlspecialchars($c['invoicing_name']) . '</GivenName>' : '')
														. (isset($c['invoicing_surname']) && $c['invoicing_surname'] != '' ? '<Surname>' . htmlspecialchars($c['invoicing_surname']) . '</Surname>' : '')
												. '</PersonName>' : '')
										. (isset($c['invoicing_tax_number']) && $c['invoicing_tax_number'] != '' ? '<TaxNumber>' . htmlspecialchars($c['invoicing_tax_number']) . '</TaxNumber>' : '')
										. (isset($c['invoicing_vat_number']) && $c['invoicing_vat_number'] != '' ? '<VatNumber>' . htmlspecialchars($c['invoicing_vat_number']) . '</VatNumber>' : '')
										. '<Address>'
										. (isset($c['invoicing_address']) && $c['invoicing_address'] != '' ? '<AddressLine>' . htmlspecialchars($c['invoicing_address']) . '</AddressLine>' : '')
										. (isset($c['invoicing_city']) && $c['invoicing_city'] != '' ? '<CityName>' . htmlspecialchars($c['invoicing_city']) . '</CityName>' : '')
										. (isset($c['invoicing_country']) && $c['invoicing_country'] != '' ? '<CountryName>' . htmlspecialchars($c['invoicing_country']) . '</CountryName>' : '')
										. (isset($c['invoicing_zip_code']) && $c['invoicing_zip_code'] != '' ? '<PostalCode>' . htmlspecialchars($c['invoicing_zip_code']) . '</PostalCode>' : '')
										. (isset($c['invoicing_province']) && $c['invoicing_province'] != '' ? '<StateProv>' . htmlspecialchars($c['invoicing_province']) . '</StateProv>' : '')
										. '</Address>'
										. (isset($c['invoicing_pec']) && $c['invoicing_pec'] != '' ? '<Pec>' . htmlspecialchars($c['invoicing_pec']) . '</Pec>' : '')
										. (isset($c['invoicing_sdi_code']) && $c['invoicing_sdi_code'] != '' ? '<SdiCode>' . htmlspecialchars($c['invoicing_sdi_code']) . '</SdiCode>' : '')
									. '</Invoicing>';
							}

							// Insert additional remarks?
							$remarks = '';
							foreach ($quote['client'] as $k => $v)
							{
								if (($p = strpos($k, 'custom_')) !== FALSE && $p == 0)
								{
									$remarks .= '<Remark Name="' . htmlspecialchars($this->STRINGS[$this->LANG]['details_' . $k]) . '"><Text>' . $v . '</Text></Remark>';
								}
							}

							$xml = '
								<ns1:OTA_VehResRQ>
									<POS>
										<Source>
											<RequestorID Type="29" ID="' . (isset($quote['vendor']) ? $quote['vendor']['id'] : AUTH_ID) . '" MessagePassword="' . htmlspecialchars(AUTH_API_KEY) . '"><CompanyName></CompanyName></RequestorID>
										</Source>
									</POS>
									<VehResRQCore>
										<VehRentalCore PickUpDateTime="' . str_replace(' ', 'T', $_SESSION[_SI]['input']['ot']) . 'Z" '
												. 'ReturnDateTime="' . str_replace(' ', 'T', $_SESSION[_SI]['input']['it']) . 'Z">
											<PickUpLocation LocationCode="' . implode(',', $checkoutBases) . '"/>
											<ReturnLocation LocationCode="' . implode(',', $checkinBases) . '"/>
										</VehRentalCore>
										<VehPref>
											<VehMakeModel Code="' . htmlspecialchars($quote['acriss_code']) . '"/>
										</VehPref>
										<Customer>
											<Primary' . (isset($quote['client']['date_of_birth']) && $quote['client']['date_of_birth'] != '' ? ' BirthDate="' . $quote['client']['date_of_birth'] . '"' : '') . '>
												<PersonName>
													<GivenName>' . htmlspecialchars($quote['client']['name']) . '</GivenName>
													<Surname>' . htmlspecialchars($quote['client']['surname']) . '</Surname>
												</PersonName>
												<Telephone PhoneNumber="' . htmlspecialchars($quote['client']['phone']) . '" />
												<Email>' . htmlspecialchars($quote['client']['email']) . '</Email>
												<Address>
													<AddressLine>' . htmlspecialchars($quote['client']['address']) . '</AddressLine>
													<CityName>' . htmlspecialchars($quote['client']['city']) . '</CityName>
													<CountryName>' . htmlspecialchars($quote['client']['country']) . '</CountryName>
													<PostalCode>' . htmlspecialchars($quote['client']['zip_code']) . '</PostalCode>
													' . (isset($c['invoicing_province']) && $c['invoicing_province'] != '' ? '<StateProv>' . htmlspecialchars($c['invoicing_province']) . '</StateProv>' : '') . '
												</Address>
												' . (isset($quote['client']['tax_number']) && $quote['client']['tax_number'] != '' ? '<TaxNumber>' . htmlspecialchars($quote['client']['tax_number']) . '</TaxNumber>' : '') . '
												' . (isset($quote['client']['place_of_birth']) && $quote['client']['place_of_birth'] != '' ? '<BirthPlace>' . htmlspecialchars($quote['client']['place_of_birth']) . '</BirthPlace>' : '') . '
											</Primary>
										</Customer>' . $invoicing . '
										<VehicleCharges>
											' . $quote['charges'] . '
											' . (isset($quote['coverage']) ? $quote['coverage']['charges'] : '') . '
											' . (isset($quote['optionals']) ? array_reduce($quote['optionals'], function($s, $e)
											{
												return $s . (isset($e['id']) && $e['id'] ? str_repeat($e['charges'], isset($e['quantity']) ? $e['quantity'] : 1) : '');
											}, '') : '') . '
										</VehicleCharges>
										' . (isset($quote['optionals']) ?
											'<SpecialEquipPrefs>
												' . array_reduce($quote['optionals'], function($s, $e)
												{
													return $s . (isset($e['id']) && $e['id'] ? '<SpecialEquipPref EquipType="' . htmlspecialchars($e['id']) . '"'
															. (isset($e['quantity']) ? ' Quantity="' . $e['quantity'] . '"' : '') . ' />' : '');
												}, '') . '
											</SpecialEquipPrefs>' : '') . '
										<TotalCharge CurrencyCode="EUR" RateTotalAmount="' . htmlspecialchars($quote['rate_vat_excl']) . '"
												EstimatedTotalAmount="' . $totals . '" />
									</VehResRQCore>
									<VehResRQInfo' . ($mippApiKey ? ' ResStatus="Initiate"' : '') . '>
										' . (isset($quote['client']['flight_no']) && strlen($quote['client']['flight_no']) > 0 ?
											'<ArrivalDetails TransportationCode="14" Number="' . htmlspecialchars($quote['client']['flight_no']) . '"/>' : '') . '
										' . (isset($quote['coverage']) ?
											'<CoveragePrefs>
												<CoveragePref Code="' . htmlspecialchars($quote['coverage']['id']) . '" CoverageType="21"/>
											</CoveragePrefs>' : '') . '
										' . $remarks . '
									</VehResRQInfo>
								</ns1:OTA_VehResRQ>';

							if (!$error)
							{
								$client = new SoapClient($wsdl, array('trace' => 1, 'cache_wsdl' => WSDL_CACHE_NONE));
								$result = $client->OTA_VehRes(new SoapVar($xml, XSD_ANYXML));

								if ($result && isset($result->Success) && $result->Success)
								{
									$cleanXml = str_ireplace(['SOAP-ENV:', 'SOAP:', 'ns1:'], '', $client->__getLastResponse());
									$xml = simplexml_load_string($cleanXml);
									$result = $xml->Body->OTA_VehResRS->VehResRSCore->VehReservation;

									unset($_SESSION[_SI]['quotes_result']);

									$key = $result['ReservationStatus'] && $result['ReservationStatus'] == 'Requested' ? 'quote' : 'reservation';

									unset($_SESSION[_SI]['quote']);
									unset($_SESSION[_SI]['reservation']);

									$_SESSION[_SI][$key] = array('data' => $quote);
									$_SESSION[_SI][$key]['totals'] = $totals;
									$_SESSION[_SI][$key]['client'] = $quote['client'];
									$_SESSION[_SI][$key]['client_name'] = $quote['client']['name'] . ' ' . $quote['client']['surname'];
									$_SESSION[_SI][$key]['client_email'] = $quote['client']['email'];

									if ($key == 'reservation')
									{
										$perc = isset($quote['vendor']) && isset($quote['vendor']['dpay']) ? $quote['vendor']['dpay'] : 1;
										$toPay = $totals;

										if (is_numeric($perc) && $perc > 0)
										{
											$toPay = sprintf('%.02f', $totals * $perc);
										}

										$_SESSION[_SI][$key]['totals_to_pay'] = $toPay;
									}

									foreach ($result->VehSegmentCore->ConfID as $k => $v)
									{
										if ($v['Type'] && $v['Type'] == 16)
										{
											$_SESSION[_SI][$key]['uid'] = (string) $v['ID'];
										}
										else if ($v['Type'] && ($v['Type'] == 14 || $v['Type'] == 23 || $v['Type'] == 39))
										{
											$_SESSION[_SI][$key]['ref_id'] = (string) $v['ID'];
										}
									}

									$res['result'] = $_SESSION[_SI][$key];
									$res['success'] = TRUE;
								}
								else
								{
									$cleanXml = str_ireplace(['SOAP-ENV:', 'SOAP:', 'ns1:'], '', $client->__getLastResponse());
									$xml = simplexml_load_string($cleanXml);
									$error = isset($xml->Body->OTA_VehResRS->Errors->Error[0]['ShortText']) ?
											$xml->Body->OTA_VehResRS->Errors->Error[0]['ShortText'] :
											(string) $xml->Body->OTA_VehResRS->Errors->Error[0];

									$error = 'Error 25002 - ' . $error;

// 									$error .= '<pre>' . htmlspecialchars($client->__getLastRequest()) . '</pre>';
// 									$error .= '<pre>' . htmlspecialchars($client->__getLastResponse()) . '</pre>';
								}
							}
						}
						else if (!$error)
						{
							$error = 'Error 25003 - ' . $params['ob'] . ' :: ' . (isset($params['ib']) ? $params['ib'] : 'NA');
						}
					}
				}
				else
				{
					$error = 'Error 20000 - Session expired';
// 					$error .= '<pre>' . print_r([$_SESSION, $params], TRUE) . '</pre>';
				}
			}
			catch (Exception $e)
			{
				$string = '';
				if ($client)
				{
					$xml = $client->__getLastResponse();

					if ($xml && strlen($xml) > 0)
					{
						$dom = new DOMDocument();
						$dom->preserveWhiteSpace = false;
						$dom->formatOutput = true;
						@$dom->loadXML($xml);

						$string = $dom->saveXML();
						if (FALSE && $string && strlen(trim($string)) > 0)
						{
							$string = '<pre>' . $string . '</pre>';
						}
						else
						{
							$string = '<pre>' . htmlspecialchars($xml) . '</pre>';
						}
					}

				}

				$error = 'Error 20001';
// 				$error .= '<pre>' . htmlspecialchars(print_r($e, TRUE)) . '</pre>'
// 						. ($client ? '<pre>' . htmlspecialchars($client->__getLastRequest()) . '</pre>' : '') . $string;
			}
		}
		else
		{
			$error = 'Error 200002';
		}

		if ($error)
		{
			$res = array('error' => $error, 'success' => FALSE);
		}

		if ($noOutput)
		{
			return $res;
		}
		else
		{
			die(json_encode($res));
		}
	}

	function postInit($attr)
	{
		if (isset($attr['lang']) && isset($this->STRINGS[$attr['lang']]))
		{
			$this->LANG = $attr['lang'];
		}

		foreach ($this->STRINGS as $k => $v)
		{
			$keys = array_keys($v);
			for ($kk = count($keys) - 1; $kk >= 0; $kk--)
			{
				if (($p = strpos($keys[$kk], 'otabo_')) !== FALSE && $p == 0)
				{
					unset($this->STRINGS[$k][$keys[$kk]]);
				}
			}
		}
	}

	function postBookingCoverages($attr)
	{
		$this->postInit($attr);
		$strings = $this->STRINGS[$this->LANG];//array_map(function($e) { return str_replace('\'', '\\\'', $e); }, $this->STRINGS[$this->LANG]);

		$quote = NULL;
		if (isset($_SESSION[_SI]) && isset($_SESSION[_SI]['quotes_result'])
				&& isset($_SESSION[_SI]['quotes_result']['quotes'])
				&& array_key_exists('qid', $_GET))
		{
			if (strpos($_GET['qid'], '.') === FALSE)
			{
				$quote = isset($_SESSION[_SI]['quotes_result']['quotes'][$_GET['qid']]) ?
						$_SESSION[_SI]['quotes_result']['quotes'][$_GET['qid']] : NULL;
			}
			else
			{
				$p = strpos($_GET['qid'], '.');
				$qid = substr($_GET['qid'], 0, $p);
				$o = substr($_GET['qid'], $p + 1);

				$quote = isset($_SESSION[_SI]['quotes_result']['quotes'][$qid])
						&& isset($_SESSION[_SI]['quotes_result']['quotes'][$qid]['options'])
						&& isset($_SESSION[_SI]['quotes_result']['quotes'][$qid]['options'][$o]) ?
						$_SESSION[_SI]['quotes_result']['quotes'][$qid]['options'][$o] : NULL;
			}
		}

		if (!$quote)
		{
			return '<script>window.location = \'' . (isset($_SESSION['quotes_result']) ? OTABO_RESULTS_URL : OTABO_FORM_URL) . '\'; </script>';
		}

		$form = '

		<div class="otabo">
			<script>

				var _S = ' . json_encode($strings) . ';
				jQuery(function( $ )
				{
					OTABO_GW_URL = \'' . admin_url('admin-ajax.php') . '\';
					OTABO_IMG_DIR = \'' . plugin_dir_url(__FILE__) . 'images/\';
					OTABO_LANG = \'' . $this->LANG . '\';
					OTABO_OID_QTY = \'' . OTABO_OID_QTY . '\';
					OTABO_OID_SEP = \'' . OTABO_OID_SEP . '\';
					OTABO_PARAMS = ' . json_encode($this->PARAMS) . ';

					OTABO_BOOKING_COVERAGES_URL = \'' . OTABO_BOOKING_COVERAGES_URL . '\'
					OTABO_BOOKING_OPTIONALS_URL = \'' . OTABO_BOOKING_OPTIONALS_URL . '\'
					OTABO_BOOKING_URL = \'' . OTABO_BOOKING_URL . '\'
					OTABO_CONFIRM_URL = \'' . OTABO_CONFIRM_URL . '\'
					OTABO_FORM_URL = \'' . OTABO_FORM_URL . '\'
					OTABO_PAYMENT = ' . json_encode(OTABO_PAYMENT) . '
					OTABO_RESULTS_URL = \'' . OTABO_RESULTS_URL . '\'

					OTABO_TERMS_URL = \'' . get_option('otabo_terms_url') . '\';

					otaboLoadBookingCoverages(' . json_encode($quote) . ', '
							. json_encode(isset($_GET['cid']) ? '' . $_GET['cid'] : NULL) . ');
				});

			</script>

			<div id="otabo_coverages">
			</div>

		</div>';

		return $form;
	}

	function postBookingDetails($attr)
	{
		$this->postInit($attr);
		$strings = $this->STRINGS[$this->LANG];//array_map(function($e) { return str_replace('\'', '\\\'', $e); }, $this->STRINGS[$this->LANG]);

		$error = isset($_GET['mpe']) ? $_GET['mpe'] : NULL;
		$mpi = isset($_GET['mpi']) ? $_GET['mpi'] : NULL;
		$osd = isset($_GET['osd']) ? $_GET['osd'] : NULL;
		$rid = isset($_GET['rid']) ? $_GET['rid'] : NULL;

		$quote = NULL;
		$reservation = NULL;

		if ($rid && $mpi)
		{
			$res = $this->ajax(['cmd' => 'load', 'rid' => $rid, 'mpi' => $mpi]);
			if ($res && isset($res['reservation']) && isset($res['reservation']['status']) && $res['reservation']['status'] == 'Initiate'
					&& isset($res['payment']))
			{
				define('OTABO_PAYMENT', TRUE);

				if (isset($res['payment']['status']) && ($res['payment']['status'] == 'auth' || $res['payment']['status'] == 'confirmed'))
				{
					$params =
					[
						'cmd' => 'confirm',
						'reservation' => json_encode($res['reservation']),
						'payment' => json_encode($res['payment'])
					];

					$confirmRes = $this->ajax($params);
					if ($confirmRes && isset($confirmRes['success']) && $confirmRes['success'])
					{
						$res = $confirmRes;
						if (isset($osd))
						{
							if (!isset($_SESSION[_SI]))
							{
								$_SESSION[_SI] = [];
							}
							else if (isset($_SESSION[_SI]['reservation']) && isset($_SESSION[_SI]['reservation']['client']))
							{
								$res['reservation']['client'] = $_SESSION[_SI]['reservation']['client'];
							}

							$_SESSION[_SI]['quote'] = NULL;
							$_SESSION[_SI]['reservation'] = $res['reservation'];
						}
					}
					else
					{
						if (isset($_SESSION[_SI]['quotes_result']))
						{
							unset($_SESSION[_SI]['quotes_result']);
						}

						return '<div class="otabo"><div id="otabo_error" style="display: block;">
								<p id="otabo_error_msg">' . $strings['booking_reservation_error'] . '</p>
								<a href="' . OTABO_RESULTS_URL . '"><button id="otabo_cancel">' . $strings['booking_backToSearch'] . '</button></a>
							</div></div>';
					}
				}
				else if (isset($osd))
				{
					if (!isset($_SESSION[_SI]))
					{
						$_SESSION[_SI] = [];
					}
					else if (isset($_SESSION[_SI]['reservation']) && isset($_SESSION[_SI]['reservation']['client']))
					{
						$res['reservation']['client'] = $_SESSION[_SI]['reservation']['client'];
					}

					$_SESSION[_SI]['quote'] = NULL;
					$_SESSION[_SI]['reservation'] = $res['reservation'];
				}

				$reservation = $res;
			}
			else if ($res && isset($res['reservation']) && isset($res['reservation']['status']) && $res['reservation']['status'] == 'Reserved'
					&& isset($res['payment']) && isset($res['payment']['status']) && ($res['payment']['status'] == 'auth' || $res['payment']['status'] == 'confirmed'))
			{
				define('OTABO_PAYMENT', TRUE);

				if (isset($osd))
				{
					$data = NULL;
					if (!isset($_SESSION[_SI]))
					{
						$_SESSION[_SI] = [];
					}
					else if (isset($_SESSION[_SI]['reservation']) && isset($_SESSION[_SI]['reservation']['client']))
					{
						$res['reservation']['client'] = $_SESSION[_SI]['reservation']['client'];
					}

					$_SESSION[_SI]['quote'] = NULL;
					$_SESSION[_SI]['reservation'] = $res['reservation'];

					$reservation = $res;
				}
			}
			else if ($res && isset($res['reservation']) && isset($res['reservation']['status']) && $res['reservation']['status'] == 'Cancelled')
			{
				// Any payment to cancel?
				if (isset($res['payment']) && isset($res['payment']['status']) && ($res['payment']['status'] == 'auth' || $res['payment']['status'] == 'confirmed'))
				{
					$params =
					[
						'cmd' => 'cancel',
						'reservation' => json_encode($res['reservation']),
						'payment' => json_encode($res['payment'])
					];

					$cancelRes = $this->ajax($params);
				}

				if (isset($_SESSION[_SI]['quotes_result']))
				{
					unset($_SESSION[_SI]['quotes_result']);
				}

				return '<div class="otabo"><div id="otabo_error" style="display: block;">
						<p id="otabo_error_msg">' . $strings['booking_reservation_error'] . '</p>
						<a href="' . OTABO_RESULTS_URL . '"><button id="otabo_cancel">' . $strings['booking_backToSearch'] . '</button></a>
					</div></div>';
			}
			else if (!defined('OTABO_PAYMENT'))
			{
				define('OTABO_PAYMENT', FALSE);
			}
		}
		else if (isset($_SESSION[_SI]) && isset($_SESSION[_SI]['quotes_result'])
				&& isset($_SESSION[_SI]['quotes_result']['quotes'])
				&& array_key_exists('qid', $_GET))
		{
			if (strpos($_GET['qid'], '.') === FALSE)
			{
				$quote = isset($_SESSION[_SI]['quotes_result']['quotes'][$_GET['qid']]) ?
						$_SESSION[_SI]['quotes_result']['quotes'][$_GET['qid']] : NULL;
			}
			else
			{
				$p = strpos($_GET['qid'], '.');
				$qid = substr($_GET['qid'], 0, $p);
				$o = substr($_GET['qid'], $p + 1);

				$quote = isset($_SESSION[_SI]['quotes_result']['quotes'][$qid])
						&& isset($_SESSION[_SI]['quotes_result']['quotes'][$qid]['options'])
						&& isset($_SESSION[_SI]['quotes_result']['quotes'][$qid]['options'][$o]) ?
						$_SESSION[_SI]['quotes_result']['quotes'][$qid]['options'][$o] : NULL;
			}
		}

		if (!$quote && !$rid || $rid && (!isset($_SESSION[_SI]) || !isset($_SESSION[_SI]['reservation'])))
		{
			return '<script>window.location = \'' . (isset($_SESSION[_SI]['quotes_result']) ? OTABO_RESULTS_URL : OTABO_FORM_URL) . '\'; </script>';
		}

		$totals = NULL;
		if ($quote)
		{
			$totals = $quote['rate'];

			// Got any coverages?
			if (isset($_GET['cid']) && $_GET['cid'] != '')
			{
				$totals += $quote['coverages'][$_GET['cid']]['rate'];
			}

			foreach ($quote['optionals'] as $k => $v)
			{
				if (isset($v['required']) && $v['required'])
				{
					$totals += $v['rate'];
				}
			}

			if (isset($_GET['oid']) && $_GET['oid'] != '')
			{
				foreach (explode(',', $_GET['oid']) AS $o)
				{
					$qty = NULL;
					if (($p = strpos($o, OTABO_OID_SEP)) !== FALSE)
					{
						$qty = substr($o, 0, $p);
						$o = substr($o, $p + strlen(OTABO_OID_SEP));
					}

					$totals += $quote['optionals'][$o]['rate'] * ($qty === NULL ? 1 : $qty);
				}
			}
		}

		$fields = '';
		$required = array_filter(array_map('trim', explode(',', get_option('otabo_booking_fields_mandatory'))));

		foreach (explode(',', get_option('otabo_booking_fields'))  as $field)
		{
			$field = trim($field);
			$fw = FALSE;

			if (($p = strpos($field, 'full_')) !== FALSE && $p == 0)
			{
				$fw = TRUE;
				$field = substr($field, 5);
			}

			if ($field == '')
			{
				$fields .= '<br />';
			}
			else if (strlen($field) > 2 && substr($field, 0, 1) == '"' && substr($field, -1) == '"')
			{
				$fields .= '<h2>' . $strings['details_' . substr($field, 1, strlen($field) - 2)] . '</h2>';
			}
			else if ($field != 'country')
			{
				$fields .= '<div' . ($fw ? ' class="w100"' : '') . '>'
						. '<label for="otabo_' . $field . '">' . $strings['details_' . $field] . '</label>'
						. '<input type="text" name="otabo_' . $field . '" id="otabo_' . $field . '" />'
					. '</div>';
			}
			else
			{
				$fields .= '<div' . ($fw ? ' class="w100"' : '') . '>'
						. '<label for="otabo_country">' . $strings['details_country'] . '</label>'
						. '<div class="input">
							<input type="text" id="otabo_country" />
							<ul id="otabo_country_list">
							</ul>
						</div>'
					. '</div>';
			}
		}

		if ($this->PARAMS['booking_fields_invoicing'])
		{
			$fields .= '<h2>' . $strings['details_invoicing'] . '</h2>';
			if (isset($strings['details_invoicing_desc']) && strlen($strings['details_invoicing_desc']) > 0)
			{
				$fields .= '<p>' . $strings['details_invoicing_desc'] . '</p>';
			}

			foreach (explode(',', 'type,company,name,surname,vat_number,tax_number,address,city,zip_code,province,full_country,pec,sdi_code')  as $field)
			{
				$field = trim($field);
				$fw = FALSE;

				if (($p = strpos($field, 'full_')) !== FALSE && $p == 0)
				{
					$fw = TRUE;
					$field = substr($field, 5);
				}

				if ($field != 'country' && $field != 'type')
				{
					$fields .= '<div' . ($fw ? ' class="w100"' : '') . '>'
							. '<label for="otabo_invoicing_' . $field . '">' . $strings['details_invoicing_' . $field] . '</label>'
							. '<input type="text" name="otabo_invoicing_' . $field . '" id="otabo_invoicing_' . $field . '" />'
						. '</div>';
				}
				else
				{
					$fields .= '<div' . ($fw ? ' class="w100"' : '') . '>'
							. '<label for="otabo_invoicing_' . $field . '">' . $strings['details_invoicing_' . $field] . '</label>'
							. '<div class="input">
								<input type="text" id="otabo_invoicing_' . $field . '" />
								<ul id="otabo_invoicing_' . $field . '_list">
								</ul>
							</div>'
						. '</div>';
				}
			}
		}

		$totalsToPay = $totals;
		if (OTABO_PAYMENT)
		{
			if ($reservation && isset($reservation['totals_to_pay']))
			{
				$totalsToPay = $reservation['totals_to_pay'];
			}
			else if ($quote)
			{
				$perc = isset($quote['vendor']) && isset($quote['vendor']['dpay']) ? $quote['vendor']['dpay'] : 1;
				$totalsToPay = sprintf('%.02f', $totalsToPay * $perc);
			}
		}

		$privacy = $quote && isset($quote['vendor']) ? $quote['vendor']['privacy_url'] : get_option('otabo_privacy_url');
		$privacy = $privacy && strlen($privacy) > 0 ? '<a target="_blank" href="' . $privacy . '">' . $strings['booking_privacy'] . '</a>' : $strings['booking_privacy'];
		$privacy = str_replace('{privacy}', $privacy, $strings['booking_privacy_text']);

		$terms = $quote && isset($quote['vendor']) ? $quote['vendor']['terms_url'] : get_option('otabo_terms_url');
		$terms = $terms && strlen($terms) > 0 ? '<a target="_blank" href="' . $terms . '">' . $strings['booking_terms'] . '</a>' : $strings['booking_terms'];
		$terms = str_replace('{terms}', $terms, $strings['booking_terms_text']);

		$form = '

		<div class="otabo">
			<script>

				var _S = ' . json_encode($strings) . ';
				jQuery(function( $ )
				{
					OTABO_GW_URL = \'' . admin_url('admin-ajax.php') . '\';
					OTABO_IMG_DIR = \'' . plugin_dir_url(__FILE__) . 'images/\';
					OTABO_LANG = \'' . $this->LANG . '\';
					OTABO_OID_QTY = \'' . OTABO_OID_QTY . '\';
					OTABO_OID_SEP = \'' . OTABO_OID_SEP . '\';
					OTABO_PARAMS = ' . json_encode($this->PARAMS) . ';

					OTABO_BOOKING_COVERAGES_URL = \'' . OTABO_BOOKING_COVERAGES_URL . '\'
					OTABO_BOOKING_OPTIONALS_URL = \'' . OTABO_BOOKING_OPTIONALS_URL . '\'
					OTABO_BOOKING_URL = \'' . OTABO_BOOKING_URL . '\'
					OTABO_CONFIRM_URL = \'' . OTABO_CONFIRM_URL . '\'
					OTABO_FORM_URL = \'' . OTABO_FORM_URL . '\'
					OTABO_PAYMENT = ' . json_encode(OTABO_PAYMENT) . '
					OTABO_RESULTS_URL = \'' . OTABO_RESULTS_URL . '\'

					' . ($rid ?
					'document.getElementById(\'otabo_submit\').innerHTML = _S.booking_loading;
					otaboLoadBooking(' . json_encode($reservation) . ', ' . json_encode($osd) . ', ' . json_encode($rid) . ', ' . json_encode($mpi) . ', ' . json_encode($error) . ', ' . json_encode($this->COUNTRIES) . ');' :

					'document.getElementById(\'otabo_submit\').innerHTML = OTABO_PAYMENT ?
							_S.booking_reservation.replace(\'{totals}\', otaboFormatCurrency(' . $totalsToPay . ')) :
							_S.booking_quote.replace(\'{totals}\', otaboFormatCurrency(' . $totals . '));
					otaboLoadBookingDetails(' . json_encode($_GET['osd']) . ', ' . json_encode($quote) . ', '
							. json_encode(isset($_GET['cid']) && $_GET['cid'] != '' ? $_GET['cid'] : NULL) . ', '
							. json_encode(isset($_GET['oid']) ? '' . $_GET['oid'] : NULL) . ', '
							. json_encode($this->COUNTRIES) . ', '
							. json_encode($required) . ');') . '
				});

			</script>

			<div id="otabo_booking">
				<fieldset id="otabo_booking_form"' . ($totals ? '' : ' style="display: none;"') . '>
					' . $fields . '
				</fieldset>

				<div class="input">
					<input id="otabo_terms" type="checkbox" /><span>' . $terms . '</span>
				</div>
				<div class="input">
					<input id="otabo_privacy" type="checkbox" /><span>' . $privacy . '</span>
				</div>

				<div id="otabo_error">
					<p id="otabo_error_msg"></p>
				</div>

				<div id="otabo_confirmed">
					<p id="otabo_confirmed_msg"></p>
				</div>

				<button id="otabo_submit"></button>
				<button id="otabo_cancel" style="display: none">' . $this->STRINGS[$this->LANG]['label_cancel'] . '</button>

				<div class="loading" id="otabo_loading">
					<img class="loading_1" src="' . plugin_dir_url(__FILE__) . 'images/loading_dot.png" />
					<img class="loading_2" src="' . plugin_dir_url(__FILE__) . 'images/loading_dot.png" />
					<img class="loading_3" src="' . plugin_dir_url(__FILE__) . 'images/loading_dot.png" />
				</div>
			</div>

		</div>';

		return $form;
	}

	function postBookingNo($attr)
	{
		$this->postInit($attr);
		$strings = $this->STRINGS[$this->LANG];//array_map(function($e) { return str_replace('\'', '\\\'', $e); }, $this->STRINGS[$this->LANG]);

		$data = isset($_SESSION[_SI]) && isset($_SESSION[_SI]['quote']) && isset($_SESSION[_SI]['quote']['ref_id']) ? $_SESSION[_SI]['quote'] :
				(isset($_SESSION[_SI]) && isset($_SESSION[_SI]['reservation']) && isset($_SESSION[_SI]['reservation']['ref_id']) ? $_SESSION[_SI]['reservation'] : NULL);
		if (!$data)
		{
 			return '<script>window.location = \'' . (isset($_SESSION['quotes_result']) ? OTABO_RESULTS_URL : OTABO_FORM_URL) . '\'; </script>';
		}

		return $data['ref_id'];
	}

	function postBookingOptionals($attr)
	{
		$this->postInit($attr);
		$strings = $this->STRINGS[$this->LANG];//array_map(function($e) { return str_replace('\'', '\\\'', $e); }, $this->STRINGS[$this->LANG]);

		$quote = NULL;
		if (isset($_SESSION[_SI]) && isset($_SESSION[_SI]['quotes_result'])
				&& isset($_SESSION[_SI]['quotes_result']['quotes'])
				&& array_key_exists('qid', $_GET))
		{
			if (strpos($_GET['qid'], '.') === FALSE)
			{
				$quote = isset($_SESSION[_SI]['quotes_result']['quotes'][$_GET['qid']]) ?
						$_SESSION[_SI]['quotes_result']['quotes'][$_GET['qid']] : NULL;
			}
			else
			{
				$p = strpos($_GET['qid'], '.');
				$qid = substr($_GET['qid'], 0, $p);
				$o = substr($_GET['qid'], $p + 1);

				$quote = isset($_SESSION[_SI]['quotes_result']['quotes'][$qid])
						&& isset($_SESSION[_SI]['quotes_result']['quotes'][$qid]['options'])
						&& isset($_SESSION[_SI]['quotes_result']['quotes'][$qid]['options'][$o]) ?
						$_SESSION[_SI]['quotes_result']['quotes'][$qid]['options'][$o] : NULL;
			}
		}

		if (!$quote)
		{
			return '<script>window.location = \'' . (isset($_SESSION['quotes_result']) ? OTABO_RESULTS_URL : OTABO_FORM_URL) . '\'; </script>';
		}

		$form = '

		<div class="otabo">
			<script>

				var _S = ' . json_encode($strings) . ';
				jQuery(function( $ )
				{
					OTABO_GW_URL = \'' . admin_url('admin-ajax.php') . '\';
					OTABO_IMG_DIR = \'' . plugin_dir_url(__FILE__) . 'images/\';
					OTABO_LANG = \'' . $this->LANG . '\';
					OTABO_OID_QTY = \'' . OTABO_OID_QTY . '\';
					OTABO_OID_SEP = \'' . OTABO_OID_SEP . '\';
					OTABO_PARAMS = ' . json_encode($this->PARAMS) . ';

					OTABO_BOOKING_COVERAGES_URL = \'' . OTABO_BOOKING_COVERAGES_URL . '\'
					OTABO_BOOKING_OPTIONALS_URL = \'' . OTABO_BOOKING_OPTIONALS_URL . '\'
					OTABO_BOOKING_URL = \'' . OTABO_BOOKING_URL . '\'
					OTABO_CONFIRM_URL = \'' . OTABO_CONFIRM_URL . '\'
					OTABO_FORM_URL = \'' . OTABO_FORM_URL . '\'
					OTABO_PAYMENT = ' . json_encode(OTABO_PAYMENT) . '
					OTABO_RESULTS_URL = \'' . OTABO_RESULTS_URL . '\'

					otaboLoadBookingOptionals(' . json_encode($quote) . ', '
							. json_encode(isset($_GET['cid']) && $_GET['cid'] != '' ? $_GET['cid'] : NULL) . ', '
							. json_encode($this->OPTIONAL_GROUPS) . ', '
							. json_encode(isset($_GET['oid']) ? '' . $_GET['oid'] : NULL) . ');
				});

			</script>

			<div id="otabo_optionals">
			</div>

		</div>';

		return $form;
	}

	function postBookingSummary($attr)
	{
		$this->postInit($attr);
		$strings = $this->STRINGS[$this->LANG];//array_map(function($e) { return str_replace('\'', '\\\'', $e); }, $this->STRINGS[$this->LANG]);

		$quote = NULL;
		if (isset($_SESSION[_SI]) && isset($_SESSION[_SI]['quotes_result'])
				&& isset($_SESSION[_SI]['quotes_result']['quotes'])
				&& array_key_exists('qid', $_GET))
		{
			if (strpos($_GET['qid'], '.') === FALSE)
			{
				$quote = isset($_SESSION[_SI]['quotes_result']['quotes'][$_GET['qid']]) ?
						$_SESSION[_SI]['quotes_result']['quotes'][$_GET['qid']] : NULL;
			}
			else
			{
				$p = strpos($_GET['qid'], '.');
				$qid = substr($_GET['qid'], 0, $p);
				$o = substr($_GET['qid'], $p + 1);

				$quote = isset($_SESSION[_SI]['quotes_result']['quotes'][$qid])
						&& isset($_SESSION[_SI]['quotes_result']['quotes'][$qid]['options'])
						&& isset($_SESSION[_SI]['quotes_result']['quotes'][$qid]['options'][$o]) ?
						$_SESSION[_SI]['quotes_result']['quotes'][$qid]['options'][$o] : NULL;
			}
		}

		if (!$quote)
		{
			return '';
		}

		$bases = $_SESSION[_SG]['bases'];
		$input = $_SESSION[_SI]['input'];

		$checkoutBase = isset($bases[$input['ob']]) ? $bases[$input['ob']] : NULL;
		$checkinBase = array_key_exists('ib', $input) && strlen($input['ib']) > 0 ? (isset($bases[$input['ib']]) ? $bases[$input['ib']] : NULL) : $checkoutBase;

		$totals = $quote['rate'];

		// Got any coverages?
		$coverage = '';
		$coverageLoading = '';
		$coverageStr = '';

		if (isset($_GET['cid']) && $_GET['cid'] != '')
		{
			$coverageLoading .= 'document.getElementById(\'otabo_coverage_rate\').innerHTML = otaboFormatCurrency(\'' . $quote['coverages'][$_GET['cid']]['rate'] . '\');';
			$coverageStr .= '<span class="coverage"><table><tr><td>' . $quote['coverages'][$_GET['cid']]['name'] . '</td>'
					. '<td id="otabo_coverage_rate" class="rate"></td></tr></table></span>';
			$totals += $quote['coverages'][$_GET['cid']]['rate'];
		}

		$coverage = '<ul id="otabo_coverage_holder"' . ($coverageStr == '' ? ' style="display: none"' : '') . '>
					<li>
						<span class="input">' . $strings['booking_coverage'] . ' <a id="otabo_coverage_modify" href="' . OTABO_BOOKING_COVERAGES_URL . '">[' . strtolower($strings['label_modify']) . ']</a>' . '</span>
						' . $coverageStr . '
					</li>
				</ul>';

		// Got any optionals?
		$optionalsLoading = '';
		$optionalsStr = '';

		foreach ($quote['optionals'] as $k => $v)
		{
			if (isset($v['required']) && $v['required'])
			{
				$optionalsLoading .= 'document.getElementById(\'otabo_optionals_rate_' . $k . '\').innerHTML = otaboFormatCurrency(\'' . $quote['optionals'][$k]['rate'] . '\');';
				$optionalsStr .= '<span class="optionals"><table><tr><td>' . $quote['optionals'][$k]['name'] . '</td>'
						. '<td class="rate" id="otabo_optionals_rate_' . $k . '"></td></tr></table></span>';
				$totals += $quote['optionals'][$k]['rate'];
			}
		}

		if (isset($_GET['oid']) && $_GET['oid'] != '')
		{
			foreach (explode(',', $_GET['oid']) AS $o)
			{
				$qty = NULL;
				if (($p = strpos($o, OTABO_OID_SEP)) !== FALSE)
				{
					$qty = substr($o, 0, $p);
					$o = substr($o, $p + strlen(OTABO_OID_SEP));
				}

				$optionalsLoading .= 'document.getElementById(\'otabo_optionals_rate_' . $o . '\').innerHTML = otaboFormatCurrency(\'' . ($quote['optionals'][$o]['rate'] * ($qty === NULL ? 1 : $qty)) . '\');';
				$optionalsStr .= '<span class="optionals"><table><tr><td>' . ($qty === NULL ? '' : $qty . OTABO_OID_QTY) . $quote['optionals'][$o]['name'] . '</td>'
						. '<td class="rate" id="otabo_optionals_rate_' . $o . '"></td></tr></table></span>';
				$totals += $quote['optionals'][$o]['rate'] * ($qty === NULL ? 1 : $qty);
			}
		}

		$optionals = '<ul id="otabo_optionals_holder"' . ($optionalsStr == '' ? ' style="display: none"' : '') . '>
					<li id="otabo_optionals_list">
						<span class="input">' . $strings['booking_optionals'] . ' <a id="otabo_optionals_modify" href="' . OTABO_BOOKING_OPTIONALS_URL . '">[' . strtolower($strings['label_modify']) . ']</a>' . '</span>
						' . $optionalsStr . '
					</li>
				</ul>';

		$sumTotals = '<ul id="otabo_totals_holder"' . ($optionalsStr != '' || $coverageStr != '' ? '' : ' style="display: none"') . '>
					<li>
						<span class="input">' . $strings['booking_totals'] . '</span>
						<span class="totals">
							<table>
								<tr>
									<td>' . $strings['booking_totals_desc'] . '</td>
									<td id="otabo_totals" class="rate"></td>
								</tr>
							</table>
						</span>
					</li>
				</ul>';

		$form = '

		<div class="otabo">
			<script>

				var _S = ' . json_encode($strings) . ';
				jQuery(function( $ )
				{
					OTABO_GW_URL = \'' . admin_url('admin-ajax.php') . '\';
					OTABO_IMG_DIR = \'' . plugin_dir_url(__FILE__) . 'images/\';
					OTABO_LANG = \'' . $this->LANG . '\';
					OTABO_OID_QTY = \'' . OTABO_OID_QTY . '\';
					OTABO_OID_SEP = \'' . OTABO_OID_SEP . '\';
					OTABO_PARAMS = ' . json_encode($this->PARAMS) . ';

					OTABO_BOOKING_COVERAGES_URL = \'' . OTABO_BOOKING_COVERAGES_URL . '\'
					OTABO_BOOKING_OPTIONALS_URL = \'' . OTABO_BOOKING_OPTIONALS_URL . '\'
					OTABO_BOOKING_URL = \'' . OTABO_BOOKING_URL . '\'
					OTABO_CONFIRM_URL = \'' . OTABO_CONFIRM_URL . '\'
					OTABO_FORM_URL = \'' . OTABO_FORM_URL . '\'
					OTABO_PAYMENT = ' . json_encode(OTABO_PAYMENT) . '
					OTABO_RESULTS_URL = \'' . OTABO_RESULTS_URL . '\'

					document.getElementById(\'otabo_checkout_time\').innerHTML = otaboFormatDate(\'' . $input['ot'] . '\');
					document.getElementById(\'otabo_checkin_time\').innerHTML = otaboFormatDate(\'' . $input['it'] . '\');
					document.getElementById(\'otabo_quote_rate\').innerHTML = otaboFormatCurrency(\'' . $quote['rate'] . '\');
					' . $coverageLoading . '
					' . $optionalsLoading . '

					var e = document.getElementById(\'otabo_totals\');
					if (e)
					{
						e.innerHTML = otaboFormatCurrency(\'' . $totals . '\');
					}
				});

			</script>

			<div id="otabo_booking_header">
				<span class="summary">' . $strings['booking_summary'] . '</span>
				<ul>
					<li>
						<span class="input">' . $strings['booking_date'] . ' <a href="' . OTABO_FORM_URL . '">[' . strtolower($strings['label_modify']) . ']</a>' . '</span>
						<span class="base">' . $checkoutBase['name'] . '<span class="date" id="otabo_checkout_time"></span></span>
					</li><li>
						<span class="base">' . $checkinBase['name'] . '<span class="date" id="otabo_checkin_time"></span></span>
					</li>
				</ul>
				<ul>
					<li>
						<span class="input">' . $strings['booking_vehicle'] . ' <a href="' . OTABO_RESULTS_URL . '">[' . strtolower($strings['label_modify']) . ']</a>' . '</span>
						<span class="vehicle">
							<table>
								<tr>
									<td>' . $quote['group'] . ($quote['name'] != $quote['group'] ? '<br />' . $quote['name'] : '') . '</td>
									<td id="otabo_quote_rate" class="rate"></td>
								</tr>
							</table>
						</span>
					</li>
				</ul>
				' . $coverage . '
				' . $optionals . '
				' . $sumTotals . '
				' . (get_permalink() != $this->getI18nPageLink(get_option('otabo_booking_url')) ?
						'<a id="otabo_booking_link" href="' . OTABO_BOOKING_URL . '"><button>Checkout</button></a>' : '') . '
			</div>

		</div>';

		return $form;
	}

	function postForm($attr)
	{
		$id = $attr && isset($attr['id']) ? $attr['id'] : 'otabo_' . ((int) rand()) . '_';

		$this->postInit($attr);
		$strings = $this->STRINGS[$this->LANG];//array_map(function($e) { return str_replace('\'', '\\\'', $e); }, $this->STRINGS[$this->LANG]);

		// Personalized form classes list?
		$params = $this->PARAMS;

		if ($attr && isset($attr['form_bases']))
		{
			$v = array_filter(explode(',', trim($attr['form_bases'])));
			$v = count($v) > 0 ? $v : NULL;

			$params['form_bases'] = $v;
		}

		if ($attr && array_key_exists('form_classes', $attr))
		{
			$v = array_filter(explode(',', trim($attr['form_classes_list'])));
			$v = count($v) > 0 ? $v : NULL;

			$params['form_classes'] = strtolower($attr['form_classes']) == 'true' || strtolower($attr['form_classes']) == 'on';
		}

		if ($attr && isset($attr['form_classes_list']))
		{
			$v = array_filter(explode(',', trim($attr['form_classes_list'])));
			$v = count($v) > 0 ? $v : NULL;

			$params['form_classes_list'] = $v;
		}

		if ($attr && isset($attr['form_class']))
		{
			$params['form_class'] = $attr['form_class'];
			$params['form_classes_list'] = [$params['form_class']];
		}

		// Default time that hides the time pickers if set.
		if ($attr && isset($attr['default_checkout_time']))
		{
			$params['form_default_checkout_time'] = $attr['default_checkout_time'];
		}

		if ($attr && isset($attr['default_checkout_time_offset']))
		{
			$params['form_default_checkout_time_offset'] = $attr['default_checkout_time_offset'];
		}

		// Default time that hides the time pickers if set.
		if ($attr && isset($attr['default_checkin_time']))
		{
			$params['form_default_checkin_time'] = $attr['default_checkin_time'];
		}

		// Show/hide the young driver flag.
		if ($attr && array_key_exists('young_driver', $attr))
		{
			$params['form_young_driver'] = strtolower($attr['young_driver']) == 'true' || strtolower($attr['young_driver']) == 'on';
		}

		// Show/hide the checkin base box.
		$params['form_checkin_base_select'] = !$attr || !array_key_exists('checkin_base_select', $attr)
				|| strtolower($attr['checkin_base_select']) == 'true' || strtolower($attr['checkin_base_select']) == 'on';

		// Default checkout day offset
		if ($attr && array_key_exists('default_checkout_day_offset', $attr))
		{
			$params['form_default_checkout_day_offset'] = $attr['default_checkout_day_offset'];
		}

		// Default checkin day offset
		if ($attr && array_key_exists('default_checkin_day_offset', $attr))
		{
			$params['form_default_checkin_day_offset'] = $attr['default_checkin_day_offset'];
		}

		$osd = isset($_GET['osd']) ? $_GET['osd'] : sha1(rand() * time());

		$form = '

		<div class="otabo">
			<script>

				var _S = ' . json_encode($strings) . ';
				jQuery(function( $ )
				{
					if (OTABO_GW_URL == null)
					{
						OTABO_GW_URL = \'' . admin_url('admin-ajax.php') . '\';
						OTABO_IMG_DIR = \'' . plugin_dir_url(__FILE__) . 'images/\';
						OTABO_LANG = \'' . $this->LANG . '\';
						OTABO_OID_QTY = \'' . OTABO_OID_QTY . '\';
						OTABO_OID_SEP = \'' . OTABO_OID_SEP . '\';
						OTABO_PARAMS = ' . json_encode($params) . ';
						OTABO_OSD = \'' . $osd . '\';

						OTABO_BOOKING_COVERAGES_URL = \'' . OTABO_BOOKING_COVERAGES_URL . '\'
						OTABO_BOOKING_OPTIONALS_URL = \'' . OTABO_BOOKING_OPTIONALS_URL . '\'
						OTABO_BOOKING_URL = \'' . OTABO_BOOKING_URL . '\'
						OTABO_CONFIRM_URL = \'' . OTABO_CONFIRM_URL . '\'
						OTABO_FORM_URL = \'' . OTABO_FORM_URL . '\'
						OTABO_PAYMENT = ' . json_encode(OTABO_PAYMENT) . '
						OTABO_RESULTS_URL = \'' . OTABO_RESULTS_URL . '\'
					}

					otaboLoadForm(' . json_encode($osd) . ', ' . json_encode($id) . ', ' . json_encode($params) . ');
				});

			</script>

			<div id="' . $id . 'form_classes" class="otabo_form_classes"></div>

			<fieldset id="' . $id . 'form" class="otabo_form">

				<p>' . $strings['checkout_base'] . '</p>
				<div class="input">
					<input type="text" placeholder="' . $strings['label_loading'] . '" id="' . $id . 'checkout_base" />
					<ul id="' . $id . 'checkout_base_list">
					</ul>
				</div>
				<p id="' . $id . 'checkin_base_select_tick">
					<input type="checkbox" id="' . $id . 'checkin_base_select" />
					<span>' . $strings['checkin_base_select'] . '</span>
				</p>
				<p id="' . $id . 'checkin_base_label" style="display: none">' . $strings['checkin_base'] . '</p>
				<div class="input" style="display: none">
					<input type="text" placeholder="' . $strings['label_loading'] . '" id="' . $id . 'checkin_base" />
					<ul id="' . $id . 'checkin_base_list">
					</ul>
				</div>
				<p id="' . $id . 'young_driver_tick" style="display: none">
					<input type="checkbox" id="' . $id . 'young_driver" />
					<span>' . $strings['booking_young_driver'] . '</span>
				</p>
				<div class="calendar_holder">
					<p>' . $strings['checkout_date'] . '</p>
					<div class="input">
						<input type="text" id="' . $id . 'checkout_date" placeholder="' . $strings['label_loading'] . '" />
						<div id="' . $id . 'checkout_date_calendar">
							<img src="' . plugin_dir_url(__FILE__) . 'images/icon_arrow_left.png" id="' . $id . 'checkout_date_left"/>
							<h1 id="' . $id . 'checkout_date_title">Gennaio</h1>
							<img src="' . plugin_dir_url(__FILE__) . 'images/icon_arrow_right.png" id="' . $id . 'checkout_date_right"/>
							<table>
								<thead>
									<tr>
										<th>L</th><th>M</th><th>M</th><th>G</th><th>V</th><th>S</th><th>D</th>
									</tr>
								</thead>
								<tbody id="' . $id . 'checkout_date_table">
								</tbody>
							</table>
						</div>
					</div>

					<div class="input">
						<input type="text" placeholder="' . $strings['label_loading'] . '" id="' . $id . 'checkout_time_input" />
						<ul id="' . $id . 'checkout_time">
						</ul>
					</div>
					<p class="error" id="' . $id . 'checkout_date_error"></p>
				</div><div class="calendar_holder">
					<p>' . $strings['checkin_date'] . '</p>
					<div class="input">
						<input type="text" id="' . $id . 'checkin_date" placeholder="' . $strings['label_loading'] . '"/>
						<div id="' . $id . 'checkin_date_calendar">
							<img src="' . plugin_dir_url(__FILE__) . 'images/icon_arrow_left.png" id="' . $id . 'checkin_date_left"/>
							<h1 id="' . $id . 'checkin_date_title"></h1>
							<img src="' . plugin_dir_url(__FILE__) . 'images/icon_arrow_right.png" id="' . $id . 'checkin_date_right"/>
							<table>
								<thead>
									<tr>
										<th>L</th><th>M</th><th>M</th><th>G</th><th>V</th><th>S</th><th>D</th>
									</tr>
								</thead>
								<tbody id="' . $id . 'checkin_date_table">
								</tbody>
							</table>
						</div>
					</div>
					<div class="input">
						<input type="text" placeholder="' . $strings['label_loading'] . '" id="' . $id . 'checkin_time_input" />
						<ul id="' . $id . 'checkin_time">
						</ul>
					</div>
					<p class="error" id="' . $id . 'checkin_date_error"></p>
				</div>

				<div>
					<button id="' . $id . 'check_dates">' . $strings['label_search'] . '</button>
				</div>

				<div id="' . $id . 'error">
					<p id="' . $id . 'error_msg"></p>
				</div>

			</fieldset>

			<div class="otabo_loading loading" id="' . $id . 'loading">
				<img class="loading_1" src="' . plugin_dir_url(__FILE__) . 'images/loading_dot.png" />
				<img class="loading_2" src="' . plugin_dir_url(__FILE__) . 'images/loading_dot.png" />
				<img class="loading_3" src="' . plugin_dir_url(__FILE__) . 'images/loading_dot.png" />
			</div>


		</div>';

		return $form;
	}

	function postResults($attr)
	{
		$this->postInit($attr);
		$strings = $this->STRINGS[$this->LANG];//array_map(function($e) { return str_replace('\'', '\\\'', $e); }, $this->STRINGS[$this->LANG]);

		$ot = isset($_GET['ot']) ? new DateTime($_GET['ot'], new DateTimeZone('UTC')) : NULL;
		$it = isset($_GET['it']) ? new DateTime($_GET['it'], new DateTimeZone('UTC')) : NULL;

		if (!$_GET['osd'] && (!isset($_GET['ob']) || !$ot || !$it))
		{
			return '<script> window.location = \'' . OTABO_FORM_URL . '\'; </script>';
		}

		$form = '

		<div class="otabo">
			<script>

				var _S = ' . json_encode($strings) . ';
				jQuery(function( $ )
				{
					OTABO_GW_URL = \'' . admin_url('admin-ajax.php') . '\';
					OTABO_IMG_DIR = \'' . plugin_dir_url(__FILE__) . 'images/\';
					OTABO_LANG = \'' . $this->LANG . '\';
					OTABO_OID_QTY = \'' . OTABO_OID_QTY . '\';
					OTABO_OID_SEP = \'' . OTABO_OID_SEP . '\';
					OTABO_PARAMS = ' . json_encode($this->PARAMS) . ';

					OTABO_BOOKING_COVERAGES_URL = \'' . OTABO_BOOKING_COVERAGES_URL . '\'
					OTABO_BOOKING_OPTIONALS_URL = \'' . OTABO_BOOKING_OPTIONALS_URL . '\'
					OTABO_BOOKING_URL = \'' . OTABO_BOOKING_URL . '\'
					OTABO_CONFIRM_URL = \'' . OTABO_CONFIRM_URL . '\'
					OTABO_FORM_URL = \'' . OTABO_FORM_URL . '\'
					OTABO_PAYMENT = ' . json_encode(OTABO_PAYMENT) . '
					OTABO_RESULTS_URL = \'' . OTABO_RESULTS_URL . '\'

					' . (isset($_GET['ot']) && isset($_GET['it']) ?
							'document.getElementById(\'otabo_checkout_time\').innerHTML = otaboFormatDate(\'' . $_GET['ot'] . '\');
							document.getElementById(\'otabo_checkin_time\').innerHTML = otaboFormatDate(\'' . $_GET['it'] . '\');' : '') . '

					' . (isset($_GET['ob']) && $ot && $it ?
							'otaboLoadResults(' . json_encode($_GET['osd']) . ', '
									. json_encode($_GET['ob']) . ', '
									. json_encode($_GET['ib']) . ', '
									. json_encode($_GET['ot']) . ', '
									. json_encode($_GET['it']) . ', '
									. (isset($_GET['oc']) ? json_encode($_GET['oc']) : '\'\'') . ', '
									. (isset($_GET['yd']) ? $_GET['yd'] : 0) . ');' :
							'otaboLoadResults(' . json_encode($_GET['osd']) . ');') . '
				});

			</script>

			<div id="otabo_results_header">
				<ul>
					<li>
						<span class="input">' . $strings['results_checkout'] . '</span>
						<span class="base" id="otabo_checkout_base">' . stripslashes(isset($_GET['obs']) ? $_GET['obs'] : '') . '</span>
						<span class="date" id="otabo_checkout_time"></span>
					</li><li>
						<span class="input">' . $strings['results_checkin'] . '</span>
						<span class="base" id="otabo_checkin_base">' . stripslashes(isset($_GET['ibs']) && strlen($_GET['ibs']) > 0 ? $_GET['ibs'] : (isset($_GET['obs']) ? $_GET['obs'] : '')) . '</span>
						<span class="date" id="otabo_checkin_time"></span>
					</li>
				</ul>
			</div>

			<div id="otabo_results_groups">
			</div>

			<div id="otabo_results">
			</div>

			<div id="otabo_error">
				<p id="otabo_error_msg"></p>
			</div>

			<div class="loading" id="otabo_loading">
				<img class="loading_1" src="' . plugin_dir_url(__FILE__) . 'images/loading_dot.png" />
				<img class="loading_2" src="' . plugin_dir_url(__FILE__) . 'images/loading_dot.png" />
				<img class="loading_3" src="' . plugin_dir_url(__FILE__) . 'images/loading_dot.png" />
			</div>

		</div>';

		return $form;
	}

	function printSoapError($client)
	{
		$reqdom = new DOMDocument();
		$reqdom->preserveWhiteSpace = false;
		$reqdom->formatOutput = true;
		$reqdom->loadXML($client->__getLastRequest());

		$dom = new DOMDocument();
		$dom->preserveWhiteSpace = false;
		$dom->formatOutput = true;
		$dom->loadXML($client->__getLastResponse());

		$res = '';
		$errors = $dom->getElementsByTagName('Error');

		for ($i = 0; $i < count($errors); $i++)
		{
			$res .= ($res == '' ? '' : "\n");
			$res .= $errors[$i]->getAttribute('Code') . ' - ' . $errors[$i]->getAttribute('ShortText');
		}

		return $res;
	}

	function otabo_field_input($key)
	{
		$type = 'text';
		if ($key == 'otabo_api_key' || $key == 'otabo_mipp_api_key' || ($p = strpos($key, 'otabo_mipp_api_key_')) !== FALSE && $p == 0)
		{
			$type = 'password';
		}
		else if ($key == 'otabo_result_groups' || $key == 'otabo_form_classes' || $key == 'otabo_form_one_way_trips' || $key == 'otabo_form_young_driver'
				|| $key == 'otabo_result_full_width' || $key == 'otabo_booking_fields_invoicing')
		{
			$type = 'checkbox';
		}
		else if ($key == 'otabo_form_bases' || $key == 'otabo_form_classes_list' || $key == 'otabo_result_details' || $key == 'otabo_optionals_groups'
				|| $key == 'otabo_custom_phrases' || $key == 'otabo_booking_fields' || $key == 'otabo_booking_fields_mandatory')
		{
			$type = 'textarea';
		}

		if ($type == 'checkbox')
		{
			echo '<input type="checkbox" id="' . $key . '" name="' . $key . '" ' . (get_option($key) || get_option($key) == 'on' ? ' checked="checked"' : '') . ' />';
		}
		else if ($type == 'textarea')
		{
			echo '<textarea id="' . $key . '" name="' . $key . '">' . get_option($key) . '</textarea>';
		}
		else
		{
 			echo '<input type="' . $type . '" id="' . $key . '" name="' . $key . '" value="' . esc_attr(get_option($key)) . '" />';
 		}

 		if (isset($this->STRINGS[$this->LANG][$key . '.desc']))
 		{
 			echo '<p>' . $this->STRINGS[$this->LANG][$key . '.desc'] . '</p>';
 		}
	}

	function otabo_field_page($key)
	{
		echo '<select id="' . $key  . '" name="' . $key . '">';

		$option = get_option($key);
		if ($pages = get_pages())
		{
			foreach ($pages as $page)
			{
				echo '<option value="' . $page->ID . '" ' . selected($page->ID, $option) . '>' . $page->post_title . '</option>';
			}
		}

		echo '</select>';

		if (isset($this->STRINGS[$this->LANG][$key . '.desc']))
		{
			echo '<p>' . $this->STRINGS[$this->LANG][$key . '.desc'] . '</p>';
		}
	}

	var $settingVendors = [];
	function otabo_register_settings()
	{
		$fields = array
		(
			'otabo_auth' => array
			(
				'name' => $this->STRINGS[$this->LANG]['otabo_auth'],
				'fields' => array
				(
					'otabo_uuid', 'otabo_api_key'
				),
			),

			'otabo_payment' => array
			(
				'name' => $this->STRINGS[$this->LANG]['otabo_payments'],
				'fields' => array
				(
					'otabo_mipp_api_key'
				),
			),

			'otabo_pages' => array
			(
				'name' => $this->STRINGS[$this->LANG]['otabo_pages'],
				'fields' => array
				(
					'otabo_form_url', 'otabo_results_url', 'otabo_coverages_url', 'otabo_optionals_url', 'otabo_booking_url', 'otabo_confirm_url'
				),
			),

			'otabo_params' => array
			(
				'name' => $this->STRINGS[$this->LANG]['otabo_params'],
				'fields' => array
				(
					'otabo_hue', 'otabo_sat', 'otabo_lig', 'otabo_form_overflow_fix', 'otabo_form_bases', 'otabo_form_classes', 'otabo_form_classes_list',
					'otabo_form_one_way_trips', 'otabo_form_young_driver',
					'otabo_optionals_groups', 'otabo_result_groups', 'otabo_result_details', 'otabo_result_full_width',
					'otabo_booking_fields', 'otabo_booking_fields_mandatory', 'otabo_booking_fields_invoicing',
					'otabo_custom_phrases'
				),
			),

			'otabo_links' => array
			(
				'name' => $this->STRINGS[$this->LANG]['otabo_links'],
				'fields' => array
				(
					'otabo_terms_url', 'otabo_privacy_url',
				),
			),
		);

		// Identify all the vendors and in case there's more than one add a specific MiPP API key field for each one.
		if (!$this->settingVendors && isset($_SESSION['otabo_vendors'])
				&& isset($_SESSION['otabo_api_key']) && $_SESSION['otabo_api_key'] == sha1(get_option('otabo_uuid') . get_option('otabo_api_key')))
		{
			$this->settingVendors = json_decode($_SESSION['otabo_vendors']);
		}

		if ($this->settingVendors)
		{
			if (count($this->settingVendors) > 1)
			{
				sort($this->settingVendors);
				foreach ($this->settingVendors as $k)
				{
					$fields['otabo_payment']['fields'][] = 'otabo_mipp_api_key_' . $k;
					$this->STRINGS[$this->LANG]['otabo_mipp_api_key_' . $k] = $this->STRINGS[$this->LANG]['otabo_mipp_api_key'] . ' - ' . $k;
				}
			}
		}
		else
		{
			array_splice($fields, 1);
		}

		foreach ($fields as $k => $v)
		{
			add_settings_section($k, $v['name'], '', 'otabo-slug');

			foreach ($v['fields'] as $kk)
			{
				$sani = 'sanitize_text_field';
				if ($kk == 'otabo_form_bases' || $kk == 'otabo_form_classes_list' || $kk == 'otabo_result_details' || $kk == 'otabo_optionals_groups'
						|| $kk == 'otabo_booking_fields' || $kk == 'otabo_booking_fields_mandatory')
				{
					$sani = 'sanitize_textarea_field';
				}
				else if ($kk == 'otabo_custom_phrases')
				{
					$sani = 'wp_check_invalid_utf8';
				}

				register_setting('otabo_options', $kk, $sani);

				$type = NULL;
				if ($k == 'otabo_pages')
				{
					$type = 'otabo_field_page';
				}
				else
				{
					$type = 'otabo_field_input';
				}

				add_settings_field($kk, $this->STRINGS[$this->LANG][$kk], array($this, $type), 'otabo-slug', $k, $kk);
			}
		};
	}

	function otabo_options_page()
	{
		if (isset($_SESSION['otabo_vendors'])
				&& isset($_SESSION['otabo_api_key']) && $_SESSION['otabo_api_key'] == sha1(get_option('otabo_uuid') . get_option('otabo_api_key')))
		{
			$this->settingVendors = json_decode($_SESSION['otabo_vendors']);
		}
		if (!$this->settingVendors && get_option('otabo_uuid') !== NULL && get_option('otabo_uuid') != ''
				&& get_option('otabo_api_key') !== NULL)
		{
			// Identify all the vendors and in case there's more than one add a specific MiPP API key field for each one.
			$bases = $this->ajax(['cmd' => 'bases']);
			if ($bases && isset($bases['success']) && $bases['success'])
			{
				if (isset($_SESSION[_SG]))
				{
					unset($_SESSION[_SG]);
				}

				$this->settingVendors = [];
				foreach ($bases['base'] as $k => $v)
				{
					foreach ($v['uuid'] as $uid)
					{
						if (array_search($uid, $this->settingVendors) === FALSE)
						{
							$this->settingVendors[] = $uid;
						}
					}
				}
			}
			else
			{
				$this->settingVendors = NULL;
			}

			$_SESSION['otabo_vendors'] = json_encode($this->settingVendors);
			$_SESSION['otabo_api_key'] = sha1(get_option('otabo_uuid') . get_option('otabo_api_key'));
		}

		$this->otabo_register_settings();

		echo '
			<div class="wrap">
				<h1>' . $this->STRINGS[$this->LANG]['otabo_settings'] . '</h1>
				<form method="post" action="options.php">';

		settings_fields('otabo_options'); // settings group name
		do_settings_sections('otabo-slug'); // just a page slug

		submit_button();

		echo $this->STRINGS[$this->LANG]['otabo_footer'];
		echo '</div></form>';
	}

	function otabo_register_options_page()
	{
		add_options_page('Otabo', 'Otabo', 'manage_options', 'otabo', array($this, 'otabo_options_page'));
	}

	var $COUNTRIES = [["code"=>"IT","name"=>"ITALIA"],["code"=>"AF","name"=>"AFGHANISTAN"],["code"=>"AL","name"=>"ALBANIA"],["code"=>"DZ","name"=>"ALGERIA"],["code"=>"AD","name"=>"ANDORRA"],["code"=>"AO","name"=>"ANGOLA"],["code"=>"AI","name"=>"ANGUILLA"],["code"=>"AG","name"=>"ANTIGUA E BARBUDA"],["code"=>"AN","name"=>"ANTILLE OLANDESI"],["code"=>"SA","name"=>"ARABIA SAUDITA"],["code"=>"AR","name"=>"ARGENTINA"],["code"=>"AM","name"=>"ARMENIA"],["code"=>"AW","name"=>"ARUBA"],["code"=>"AU","name"=>"AUSTRALIA"],["code"=>"AT","name"=>"AUSTRIA"],["code"=>"AZ","name"=>"AZERBAIGIAN"],["code"=>"BS","name"=>"BAHAMAS"],["code"=>"BH","name"=>"BAHRAIN"],["code"=>"BD","name"=>"BANGLADESH"],["code"=>"BB","name"=>"BARBADOS"],["code"=>"BE","name"=>"BELGIO"],["code"=>"BZ","name"=>"BELIZE"],["code"=>"BJ","name"=>"BENIN"],["code"=>"BM","name"=>"BERMUDA"],["code"=>"BT","name"=>"BHUTAN"],["code"=>"BY","name"=>"BIELORUSSIA"],["code"=>"MM","name"=>"BIRMANIA"],["code"=>"BO","name"=>"BOLIVIA"],["code"=>"BQ","name"=>"BONAIRE"],["code"=>"BA","name"=>"BOSNIA ED ERZEGOVINA"],["code"=>"BW","name"=>"BOTSWANA"],["code"=>"BR","name"=>"BRASILE"],["code"=>"BN","name"=>"BRUNEI"],["code"=>"BG","name"=>"BULGARIA"],["code"=>"BF","name"=>"BURKINA FASO"],["code"=>"BI","name"=>"BURUNDI"],["code"=>"KH","name"=>"CAMBOGIA"],["code"=>"CM","name"=>"CAMERUN"],["code"=>"CA","name"=>"CANADA"],["code"=>"CV","name"=>"CAPO VERDE"],["code"=>"TD","name"=>"CIAD"],["code"=>"CL","name"=>"CILE"],["code"=>"CN","name"=>"CINA"],["code"=>"CY","name"=>"CIPRO"],["code"=>"VA","name"=>"CITTÃ€ DEL VATICANO"],["code"=>"CO","name"=>"COLOMBIA"],["code"=>"KM","name"=>"COMORE"],["code"=>"KP","name"=>"COREA DEL NORD"],["code"=>"KR","name"=>"COREA DEL SUD"],["code"=>"CI","name"=>"COSTA D'AVORIO"],["code"=>"CR","name"=>"COSTA RICA"],["code"=>"HR","name"=>"CROAZIA"],["code"=>"CU","name"=>"CUBA"],["code"=>"CW","name"=>"CURAÃ‡AO"],["code"=>"DK","name"=>"DANIMARCA"],["code"=>"DM","name"=>"DOMINICA"],["code"=>"EC","name"=>"ECUADOR"],["code"=>"EG","name"=>"EGITTO"],["code"=>"SV","name"=>"EL SALVADOR"],["code"=>"AE","name"=>"EMIRATI ARABI UNITI"],["code"=>"ER","name"=>"ERITREA"],["code"=>"EE","name"=>"ESTONIA"],["code"=>"ET","name"=>"ETIOPIA"],["code"=>"FJ","name"=>"FIGI"],["code"=>"PH","name"=>"FILIPPINE"],["code"=>"FI","name"=>"FINLANDIA"],["code"=>"FR","name"=>"FRANCIA"],["code"=>"GA","name"=>"GABON"],["code"=>"GM","name"=>"GAMBIA"],["code"=>"GE","name"=>"GEORGIA"],["code"=>"GS","name"=>"GEORGIA DEL SUD E ISOLE SANDWICH MERIDIONALI"],["code"=>"DE","name"=>"GERMANIA"],["code"=>"GH","name"=>"GHANA"],["code"=>"JM","name"=>"GIAMAICA"],["code"=>"JP","name"=>"GIAPPONE"],["code"=>"DJ","name"=>"GIBUTI"],["code"=>"JO","name"=>"GIORDANIA"],["code"=>"GR","name"=>"GRECIA"],["code"=>"GD","name"=>"GRENADA"],["code"=>"GL","name"=>"GROENLANDIA"],["code"=>"GP","name"=>"GUADALUPA"],["code"=>"GU","name"=>"GUAM"],["code"=>"GT","name"=>"GUATEMALA"],["code"=>"GN","name"=>"GUINEA"],["code"=>"GQ","name"=>"GUINEA EQUATORIALE"],["code"=>"GW","name"=>"GUINEA-BISSAU"],["code"=>"GY","name"=>"GUYANA"],["code"=>"GF","name"=>"GUYANA FRANCESE"],["code"=>"HT","name"=>"HAITI"],["code"=>"HN","name"=>"HONDURAS"],["code"=>"HK","name"=>"HONG KONG"],["code"=>"IN","name"=>"INDIA"],["code"=>"ID","name"=>"INDONESIA"],["code"=>"IR","name"=>"IRAN"],["code"=>"IQ","name"=>"IRAQ"],["code"=>"IE","name"=>"IRLANDA"],["code"=>"IS","name"=>"ISLANDA"],["code"=>"CX","name"=>"ISOLA DEL NATALE"],["code"=>"NF","name"=>"ISOLA NORFOLK"],["code"=>"KY","name"=>"ISOLE CAYMAN"],["code"=>"CC","name"=>"ISOLE COCOS E KEELING"],["code"=>"CK","name"=>"ISOLE COOK"],["code"=>"FK","name"=>"ISOLE FALKLAND"],["code"=>"MP","name"=>"ISOLE MARIANNE SETTENTRIONALI"],["code"=>"MH","name"=>"ISOLE MARSHALL"],["code"=>"PN","name"=>"ISOLE PITCAIRN"],["code"=>"SB","name"=>"ISOLE SALOMONE"],["code"=>"TF","name"=>"ISOLE SPARSE NELL'OCEANO INDIANO"],["code"=>"VI","name"=>"ISOLE VERGINI AMERICANE"],["code"=>"VG","name"=>"ISOLE VERGINI BRITANNICHE"],["code"=>"IL","name"=>"ISRAELE"],["code"=>"KZ","name"=>"KAZAKISTAN"],["code"=>"KE","name"=>"KENYA"],["code"=>"KG","name"=>"KIRGHIZISTAN"],["code"=>"KI","name"=>"KIRIBATI"],["code"=>"KW","name"=>"KUWAIT"],["code"=>"LA","name"=>"LAOS"],["code"=>"LS","name"=>"LESOTHO"],["code"=>"LV","name"=>"LETTONIA"],["code"=>"LB","name"=>"LIBANO"],["code"=>"LR","name"=>"LIBERIA"],["code"=>"LY","name"=>"LIBIA"],["code"=>"LI","name"=>"LIECHTENSTEIN"],["code"=>"LT","name"=>"LITUANIA"],["code"=>"LU","name"=>"LUSSEMBURGO"],["code"=>"MO","name"=>"MACAO"],["code"=>"MK","name"=>"MACEDONIA"],["code"=>"MG","name"=>"MADAGASCAR"],["code"=>"MW","name"=>"MALAWI"],["code"=>"MV","name"=>"MALDIVE"],["code"=>"MY","name"=>"MALESIA"],["code"=>"ML","name"=>"MALI"],["code"=>"MT","name"=>"MALTA"],["code"=>"MA","name"=>"MAROCCO"],["code"=>"MQ","name"=>"MARTINICA"],["code"=>"MR","name"=>"MAURITANIA"],["code"=>"MU","name"=>"MAURITIUS"],["code"=>"YT","name"=>"MAYOTTE"],["code"=>"MX","name"=>"MESSICO"],["code"=>"MD","name"=>"MOLDAVIA"],["code"=>"MN","name"=>"MONGOLIA"],["code"=>"ME","name"=>"MONTENEGRO"],["code"=>"MS","name"=>"MONTSERRAT"],["code"=>"MZ","name"=>"MOZAMBICO"],["code"=>"NA","name"=>"NAMIBIA"],["code"=>"NR","name"=>"NAURU"],["code"=>"NP","name"=>"NEPAL"],["code"=>"NI","name"=>"NICARAGUA"],["code"=>"NE","name"=>"NIGER"],["code"=>"NG","name"=>"NIGERIA"],["code"=>"NU","name"=>"NIUE"],["code"=>"NO","name"=>"NORVEGIA"],["code"=>"NC","name"=>"NUOVA CALEDONIA"],["code"=>"NZ","name"=>"NUOVA ZELANDA"],["code"=>"OM","name"=>"OMAN"],["code"=>"NL","name"=>"PAESI BASSI"],["code"=>"PK","name"=>"PAKISTAN"],["code"=>"PW","name"=>"PALAU"],["code"=>"PS","name"=>"PALESTINA"],["code"=>"PA","name"=>"PANAMA"],["code"=>"PG","name"=>"PAPUA NUOVA GUINEA"],["code"=>"PY","name"=>"PARAGUAY"],["code"=>"PE","name"=>"PERÃ™"],["code"=>"PF","name"=>"POLINESIA FRANCESE"],["code"=>"PL","name"=>"POLONIA"],["code"=>"PR","name"=>"PORTO RICO"],["code"=>"PT","name"=>"PORTOGALLO"],["code"=>"MC","name"=>"PRINCIPATO DI MONACO"],["code"=>"QA","name"=>"QATAR"],["code"=>"GB","name"=>"REGNO UNITO"],["code"=>"CZ","name"=>"REPUBBLICA CECA"],["code"=>"CF","name"=>"REPUBBLICA CENTRAFRICANA"],["code"=>"CG","name"=>"REPUBBLICA DEL CONGO"],["code"=>"CD","name"=>"REPUBBLICA DEMOCRATICA DEL CONGO"],["code"=>"DO","name"=>"REPUBBLICA DOMINICANA"],["code"=>"RE","name"=>"RIUNIONE"],["code"=>"RO","name"=>"ROMANIA"],["code"=>"RW","name"=>"RUANDA"],["code"=>"RU","name"=>"RUSSIA"],["code"=>"BQ","name"=>"SABA"],["code"=>"KN","name"=>"SAINT KITTS E NEVIS"],["code"=>"VC","name"=>"SAINT VINCENT E GRENADINE"],["code"=>"BL","name"=>"SAINT-BARTHÃ‰LEMY"],["code"=>"MF","name"=>"SAINT-MARTIN"],["code"=>"PM","name"=>"SAINT-PIERRE E MIQUELON"],["code"=>"WS","name"=>"SAMOA"],["code"=>"AS","name"=>"SAMOA AMERICANE"],["code"=>"SM","name"=>"SAN MARINO"],["code"=>"SH","name"=>"SANT'ELENA, ASCENSIONE E TRISTAN DA CUNHA"],["code"=>"LC","name"=>"SANTA LUCIA"],["code"=>"ST","name"=>"SÃO TOMÉ E PRÍNCIPE"],["code"=>"SN","name"=>"SENEGAL"],["code"=>"RS","name"=>"SERBIA"],["code"=>"SC","name"=>"SEYCHELLES"],["code"=>"SL","name"=>"SIERRA LEONE"],["code"=>"SG","name"=>"SINGAPORE"],["code"=>"BQ","name"=>"SINT EUSTATIUS"],["code"=>"SX","name"=>"SINT MAARTEN"],["code"=>"SY","name"=>"SIRIA"],["code"=>"SK","name"=>"SLOVACCHIA"],["code"=>"SI","name"=>"SLOVENIA"],["code"=>"SO","name"=>"SOMALIA"],["code"=>"ES","name"=>"SPAGNA"],["code"=>"LK","name"=>"SRI LANKA"],["code"=>"FM","name"=>"STATI FEDERATI DI MICRONESIA"],["code"=>"US","name"=>"STATI UNITI D'AMERICA"],["code"=>"ZA","name"=>"SUDAFRICA"],["code"=>"SD","name"=>"SUDAN"],["code"=>"SS","name"=>"SUDAN DEL SUD"],["code"=>"SR","name"=>"SURINAME"],["code"=>"SE","name"=>"SVEZIA"],["code"=>"CH","name"=>"SVIZZERA"],["code"=>"SZ","name"=>"SWAZILAND"],["code"=>"TJ","name"=>"TAGIKISTAN"],["code"=>"TW","name"=>"TAIWAN"],["code"=>"TZ","name"=>"TANZANIA"],["code"=>"TH","name"=>"THAILANDIA"],["code"=>"TL","name"=>"TIMOR EST"],["code"=>"TG","name"=>"TOGO"],["code"=>"TK","name"=>"TOKELAU"],["code"=>"TO","name"=>"TONGA"],["code"=>"TT","name"=>"TRINIDAD E TOBAGO"],["code"=>"TN","name"=>"TUNISIA"],["code"=>"TR","name"=>"TURCHIA"],["code"=>"TM","name"=>"TURKMENISTAN"],["code"=>"TC","name"=>"TURKS E CAICOS"],["code"=>"TV","name"=>"TUVALU"],["code"=>"UA","name"=>"UCRAINA"],["code"=>"UG","name"=>"UGANDA"],["code"=>"HU","name"=>"UNGHERIA"],["code"=>"UY","name"=>"URUGUAY"],["code"=>"UZ","name"=>"UZBEKISTAN"],["code"=>"VU","name"=>"VANUATU"],["code"=>"VE","name"=>"VENEZUELA"],["code"=>"VN","name"=>"VIETNAM"],["code"=>"WF","name"=>"WALLIS E FUTUNA"],["code"=>"YE","name"=>"YEMEN"],["code"=>"ZM","name"=>"ZAMBIA"],["code"=>"ZW","name"=>"ZIMBABWE"]];

	var $LANG = NULL;

	var $OPTIONAL_GROUPS = array
	(
		'special' => [ '13', '101' ],
		'children' => [ '7', '8', '9' ],
		'other' => []
	);

	var $PARAMS = [];

	var $STRINGS = array
	(
		'it' => array
		(
			'error' => 'Ci dispiace, sembra che il sito stia incontrando difficoltà tecniche. Prova a ricaricare la pagina o collegati di nuovo più tardi in caso che il problema persista.',

			'label_cancel' => 'Annulla',
			'label_loading' => 'Caricamento…',
			'label_modify' => 'Modifica',
			'label_search' => 'Cerca',
			'label_select' => 'Seleziona…',

			'otabo_settings' => 'Impostazioni Otabo',

			'otabo_auth' => 'Authenticazione',
			'otabo_uuid' => 'UUID',
			'otabo_uuid.desc' => 'Inserisci l\'identificatore dell\'agente/broker come visualizzato su CRS.',
			'otabo_api_key' => 'API Key',
			'otabo_api_key.desc' => 'Inserisci l\'API Key dell\'agente/broker esattamente come viene visualizzato su CRS.',

			'otabo_payments' => 'Pagamenti',
			'otabo_mipp_api_key' => 'MiPP API Key',
			'otabo_mipp_api_key.desc' => 'Inserisci l\'API Key di MiPP che ti è stato comunicato da My Appy. Senza un MiPP Api Key valido sarà possibile solo salvare dei preventivi. Con un MiPP API Key possono essere accettati dei pagamenti che ti permette di ricevere direttamente delle prenotazioni.',

			'otabo_hue' => 'Valore H del color principale in HSL',
			'otabo_sat' => 'Valore S del color principale in HSL',
			'otabo_lig' => 'Valore L del color principale in HSL',

			'otabo_custom_phrases' => 'Testi personalizzati',
			'otabo_custom_phrases.desc' => 'Usa questo campo per personalizzare le frasi di otabo. Contatta My Appy per saperne di più.<br /><b>Phrase mapping:</b><ul>
					<li>booking_base_{id}: Base names</li>
					<li>booking_clazz_{name}: Macrogroups</li>
					<li>booking_group_{name}: Collective groups</li>
					<li>booking_vehicle_{name}: Vehicle groups</li>
					<li>booking_tariff_{category name}: Tariff option</li>
					<li>booking_coverage_{id}_title: Coverage titles</li>
					<li>booking_coverage_{id}_desc: Coverage text</li>
					<li>booking_optionals_{group_id}: Optionals group id</li>
					<li>booking_optionals_{ota_eqp_id}_title: Optionals name</li>
					<li>booking_optionals_{ota_eqp_id}_desc: Optionals description</li>
				</ul>',
			'otabo_pages' => 'Pagine',
			'otabo_form_url' => 'Modulo di ricerca',
			'otabo_form_url.desc' => 'Pagina che visualizza il form di ricerca.',
			'otabo_results_url' => 'Elenco risultati',
			'otabo_results_url.desc' => 'Pagina che visualizza la lista dei risultati.',
			'otabo_coverages_url' => 'Coperture',
			'otabo_coverages_url.desc' => 'Pagina che visualizza le opzioni assicurativi.',
			'otabo_optionals_groups' => 'Gruppi per gli optionals',
			'otabo_optionals_groups.desc' => 'Usa questo campo per organizzare la pagina degli optionals. Contatta My Appy per saperne di più.',
			'otabo_optionals_url' => 'Optionals',
			'otabo_optionals_url.desc' => 'Pagina che visualizza gli optionals.',
			'otabo_booking_url' => 'Dati personali',
			'otabo_booking_url.desc' => 'Pagina che visualizza il form dove l\'utente inserisce i suoi dati personali prima di inviare il preventivo/prenotazione.',
			'otabo_confirm_url' => 'Conferma',
			'otabo_confirm_url.desc' => 'Pagina che visualizza la conferma del preventivo/prenotazione.',

			'otabo_params' => 'Parametri',
			'otabo_booking_fields' => 'Campi dati personali',
			'otabo_booking_fields.desc' => 'Usa questo campo per indicare i campi visibili sul modulo per inserire i dati personali. Contatta My Appy per saperne di più.',
			'otabo_booking_fields_mandatory' => 'Campi dati personali obbligatori',
			'otabo_booking_fields_mandatory.desc' => 'Usa questo campo per indicare i campi obbligatori del modulo per inserire i dati personali. Contatta My Appy per saperne di più.',
			'otabo_booking_fields_invoicing' => 'Campi dati fatturazione',
			'otabo_booking_fields_invoicing.desc' => 'Attiva questo flag per mostrare la sezione per l\'inserimento dei dati di fatturazione',
			'otabo_form_bases' => 'Filtro stazioni',
			'otabo_form_bases.desc' => 'Elenco delle stazioni da visualizzare, dividendoli con una virgola (","). Lascia il campo vuoto se vuoi visualizzare tutte le stazioni registrate su CRS.',
			'otabo_form_classes' => 'Visualizza schede con macrogruppi',
			'otabo_form_classes.desc' => 'Visualizza la barra sopra il modulo di ricerca che permette all\'utente di scegliere prima il macrogruppo della vettura ricercata.',
			'otabo_form_classes_list' => 'Filtro macrogruppi',
			'otabo_form_classes_list.desc' => 'Usa questo campo per filtrare e riordinare i macrogruppi visualizzati sopra il form iniziale. Scrivi i nomi dei gruppi da visualizzare esattamente come sono registrati su CRS, dividendoli con una virgola (","). Lascia il campo vuoto se vuoi utilizzare i gruppi così come sono registrati su CRS.',
			'otabo_form_one_way_trips' => 'Mostra campo per selezionare stazione di riconsegna',
			'otabo_form_one_way_trips.desc' => 'Attiva questo flag se vuoi visualizzare un campo per selezionare la stazione di riconsegna diversa da quella di ritiro.',
			'otabo_form_overflow_fix' => 'Overflow fix level',
			'otabo_form_overflow_fix.desc' => 'Se hai il problema che il container in cui si trova il form taglia gli elementi del form, usa questo campo per indicare il numero di livelli che il plugin proverà a sistemare impostando l\'overflow "visible" per i propri elementi.',
			'otabo_form_young_driver' => 'Flag young driver',
			'otabo_form_young_driver.desc' => 'Attiva questo flag se vuoi visualizzare un checkbox per indicare che il conducente è giovane.',
			'otabo_result_details' => 'Dettagli visibili',
			'otabo_result_details.desc' => 'Se vuoi mostrare altri dettagli, oltre i filtri generali come numero posti, ecc., scrivi l\'ID di ogni sezione (la trovi come parametri Otabo sulla scheda Tariffa > Web) qui, usando la virgola (",") come separatore in caso di più sezioni.',
			'otabo_result_groups' => 'Visualizza gruppi collettivi',
			'otabo_result_groups.desc' => 'Attiva questo flag per visualizzare tutti i gruppi collettivi sopra la lista con i risultati.',
			'otabo_result_full_width' => 'Visualiz. 100% larghezza',
			'otabo_result_full_width.desc' => 'Attiva questo flag per visualizzare una macchina a riga. In questo caso verranno pure raggruppati offerte diverse (se disponibili) per la stessa macchina dando l\'utente una visualizzazione più completa sulle sue scelte per una tipologia di vettura.',
			'otabo_links' => 'Links',
			'otabo_terms_url' => 'Termini e condizioni',
			'otabo_terms_url.desc' => 'Link che porta l\'utente alle pagina con i termini e condizioni.',
			'otabo_privacy_url' => 'Privacy policy',
			'otabo_privacy_url.desc' => 'Link che porta l\'utente alla pagina con la privacy policy.',

			'otabo_footer' => '<h1>Elementi da inserire sulle pagine</h1>
				<p>Tutti gli elementi forniti dal plugin possono essere aggiunti ad una pagina inserendo un breve codice con la sintassi seguente: <b>[nome elemento]</b>.
					L\'elenco seguente vi da una breve descrizione degli elementi da inserire.</p>
				<dl>
					<dt>[otabo_form]</dt>
					<dd>Stampa il modulo che permette all\'utente di scegliere la stazione di checkout/checkin e le date/orari.
					Se il flag <b>Visualizza schede con macrogruppi</b> è attivo, verrano pure visualizzate le schede con i macrogruppi disponibili.</dd>

					<dt>[otabo_results]</dt>
					<dd>Crea la lista degli risultati delle ricerca.</dd>

					<dt>[otabo_booking_coverages]</dt>
					<dd>Dopo aver selezionato una vettura dalla lista dei risultati, può selezionare una copertura assicurativa da una lista con tutte le opzioni disponibili,
					create da questo elemento.</dd>

					<dt>[otabo_booking_optionals]</dt>
					<dd>Visualizza una lista con tutti gli optionals disponibili per le date scelte.</dd>

					<dt>[otabo_booking_details]</dt>
					<dd>Dopo aver selezionato il mezzo, le coperture e gli optionals, l\'utente deve inserire i suoi dati personali per salvare il preventivo o inviare la
					prenotazione. Questo codice stampa il modulo per farlo formando l\'ultimo passaggio attivo della procedura.</dd>

					<dt>[otabo_booking_no]</dt>
					<dd>Questo codice stampa semplicemente il numero del preventivo salvato o della prenotazione effettuato. Può essere incluso in un testo confermativo sulla pagina
					di conferma, p.es. come fatto nel testo seguente: <i>Congratulazioni, la tua prenotazione [otabo_booking_no] è stata inviata con successo.</i>.</dd>

					<dt>[otabo_booking_summary]</dt>
					<dd>Genera una tabella riepilogativa visualizzando la vettura scelta, la copertura e gli optionals e l\'importo complessivo. Accanto ogni voce del riepilogo
					verrà pure stampato un link che ti porta di nuovo sulla pagina che ti permette di cambiare quello che hai scelto.</dd>
				</dl>',

			'booking_backToSearch' => 'Vai ai risultati della ricerca',
			'booking_canceled' => 'Prenotazione {ref_id} annullata',
			'booking_confirmed' => 'Prenotazione {ref_id} confermato',
			'booking_coverage' => 'Copertura',
			'booking_coverage_deposit' => 'Deposito cauzionale',
			'booking_coverage_excess' => 'Franchigia danni/furto',
			'booking_coverage_excess_damage' => 'Franchigia danni',
			'booking_coverage_excess_theft' => 'Franchigia furto',
			'booking_coverage_view_terms' => 'Leggi le condizioni generali di noleggio per ulteriori info',
			'booking_date' => 'Quando e dove',
			'booking_error_checkin_base' => 'Seleziona una località per la riconsegna.',
			'booking_error_checkin_date' => 'La località di riconsegna risulta chiusa per il giorno scelto.',
			'booking_error_checkin_date_before' => 'La data e ora della riconsegna non può precedere quella del ritiro.',
			'booking_error_checkin_date_invalid' => 'La data della riconsegna non è valida.',
			'booking_error_checkin_time' => 'Seleziona un orario tra quelli disponibili per la riconsegna.',
			'booking_error_checkout_base' => 'Seleziona una località per il ritiro.',
			'booking_error_checkout_date' => 'La località di ritiro risulta chiusa per il giorno scelto.',
			'booking_error_checkout_date_invalid' => 'La data del ritiro non è valido.',
			'booking_error_checkout_time' => 'Seleziona un orario tra quelli disponibili per il ritiro.',
			'booking_error_email_not_valid' => 'L\'indirizzo email non sembra essere valido.',
			'booking_error_emails_not_equal' => 'Gli indirizzi email inseriti non sono uguali.',
			'booking_error_fill_form' => 'Bisogna compilare tutti i campi del form per procedere.',
			'booking_error_terms' => 'Bisogna accettare i termini e le condizioni di noleggio e il trattamento dei dati personali.',
			'booking_loading' => 'Caricamento…',
			'booking_optionals' => 'Optional',
			'booking_optionals_next' => 'Continua',
			'booking_optionals_special' => 'Suggeriti per te',
			'booking_optionals_children' => 'Viaggiando con i bambini',
			'booking_optionals_other' => 'Optional aggiuntivi',
			'booking_optionals_totals' => 'Totale Noleggio',
			'booking_optionals_13_desc' => 'Noleggia un sistema di navigazione satellitare per avere a disposizione mappe aggiornate, notizie sul traffico in tempo reale, indicazioni su luoghi di interesse e navigazione audio in tutte le lingue europee più arabo, cinese, russo.',
			'booking_optionals_101_desc' => 'Condividi il piacere della guida… Aggiungi uno o più conducenti al tuo noleggio e ognuno beneficerà della copertura per danni/furto del conducente principale.',
			'booking_pay' => 'Paga adesso {totals}',
			'booking_privacy' => 'trattamento dei propri dati personali',
			'booking_privacy_text' => 'Autorizzo al {privacy} secondo il Regolamento UE n. 679 del 2016 (GDPR) e D.Lgs. n. 101 del 2018 e successive modifiche',
			'booking_quote' => 'Blocca prezzo a {totals}',
			'booking_reservation' => 'Prenota e paga adesso {totals}',
			'booking_reservation_error' => 'Ops! Hai cancellato la prenotazione o il mezzo da te selezionato è stato appena prenotato da un\'altra parte. Il tuo pagamento è stato regolarmente annullato. Fai una nuova ricerca.',
			'booking_reservation_submitting' => 'Conferma prenotazione',
			'booking_reservation_title' => 'Prenotazione n° {ref_id}',
			'booking_summary' => 'Riepilogo',
			'booking_terms' => 'i Termini e le Condizioni di noleggio',
			'booking_terms_text' => 'Desidero noleggiare il veicolo indicato secondo {terms} e alle tariffe in vigore. Ho letto ed accettato i Termini e le Condizioni di noleggio, tariffarie e assicurative.',
			'booking_totals' => 'Totale',
			'booking_totals_desc' => 'Totale noleggio',
			'booking_vehicle' => 'Veicolo',
			'booking_young_driver' => 'Il conducente ha meno di 21 anni',

			'checkout_base' => 'Località di ritiro',
			'checkout_base_select' => 'Seleziona la località di ritiro prima…',
			'checkout_date' => 'Data di ritiro',
			'checkin_base' => 'Località di riconsegna',
			'checkin_base_select' => 'Effettua la riconsegna in una località diversa',
			'checkin_date' => 'Data di riconsegna',

			'details_address' => 'Indirizzo e civico',
			'details_date_of_birth' => 'Data di nascita (GG-MM-AAAA)',
			'details_city' => 'Città',
			'details_country' => 'Nazione',
			'details_email' => 'Indirizzo email',
			'details_email_confirm' => 'Conferma indirizzo email',
			'details_flight_no' => 'Numero volo',
			'details_name' => 'Nome',
			'details_phone' => 'Numero di telefono',
			'details_place_of_birth' => 'Luogo di nascita',
			'details_province' => 'Sigla Provincia (XX)',
			'details_surname' => 'Cognome',
			'details_tax_number' => 'Codice fiscale',
			'details_zip_code' => 'CAP',

			'details_invoicing' => 'Dati Fatturazione',
			'details_invoicing_desc' => '',
			'details_invoicing_address' => 'Indirizzo e civico',
			'details_invoicing_city' => 'Città',
			'details_invoicing_company' => 'Ragione sociale',
			'details_invoicing_country' => 'Nazione',
			'details_invoicing_name' => 'Nome',
			'details_invoicing_pec' => 'PEC',
			'details_invoicing_province' => 'Sigla provincia (XX)',
			'details_invoicing_sdi_code' => 'Codice SdI',
			'details_invoicing_surname' => 'Cognome',
			'details_invoicing_tax_number' => 'Codice fiscale',
			'details_invoicing_type' => 'Tipologia',
			'details_invoicing_type_business' => 'Azienda',
			'details_invoicing_type_one_man_business' => 'Dittà individuale',
			'details_invoicing_type_private' => 'Privato',
			'details_invoicing_type_public_amm' => 'Pubblica amministrazione',
			'details_invoicing_vat_number' => 'Partita IVA',
			'details_invoicing_zip_code' => 'CAP',

			'format.datetime' => 'd-m-Y H:i',

			'month_0' => 'Gennaio',
			'month_1' => 'Febbraio',
			'month_2' => 'Marzo',
			'month_3' => 'Aprile',
			'month_4' => 'Maggio',
			'month_5' => 'Giugno',
			'month_6' => 'Luglio',
			'month_7' => 'Agosto',
			'month_8' => 'Settembre',
			'month_9' => 'Ottobre',
			'month_10' => 'Novembre',
			'month_11' => 'Dicembre',

			'param_ChmCharacteristics' => 'Characteristics',
			'param_ChmCharacteristics_Engine' => 'Motore',
			'param_ChmCharacteristics_LengthOverall' => 'L.O.A.',
			'param_ChmCharacteristics_LengthOverall_Unit' => 'm',
			'param_ChmCharacteristics_TankSize' => 'Serbatoio Diesel',
			'param_ChmCharacteristics_TankSize_Unit' => 'L',
			'param_ChmCharacteristics_Type' => 'Tipo',
			'param_ChmCharacteristics_WaterTankSize' => 'Serbatoio Acqua',
			'param_ChmCharacteristics_WaterTankSize_Unit' => 'L',
			'param_ChmCharacteristics_Year' => 'Anno',

			'param_ChmEquipment' => 'Equipment',
			'param_ChmEquipment_BowThruster' => 'Bow thruster',
			'param_ChmEquipment_BowThruster_1' => 'Sì',
			'param_ChmEquipment_BowThruster_0' => 'No',
			'param_ChmEquipment_Gps' => 'GPS',
			'param_ChmEquipment_Gps_bg_vulcan' => 'GPS B&G Vulcan',
			'param_ChmEquipment_Gps_gps_plotter' => 'GPS Plotter',
			'param_ChmEquipment_Gps_navionics' => 'iPad (Navionics App)',
			'param_ChmEquipment_Mainsail' => 'Mainsail',
			'param_ChmEquipment_Mainsail_furling' => 'Furling',
			'param_ChmEquipment_Mainsail_lazy_bag' => 'Lazy bag',
			'param_ChmEquipment_Teak' => 'Teak',
			'param_ChmEquipment_Teak_cockpit' => 'Cockpit',
			'param_ChmEquipment_Teak_full' => 'Full',
			'param_ChmEquipment_Teak_none' => 'None',

			'param_VehicleFilters_BerthsNo' => 'Cuccette',
			'param_VehicleFilters_BigBags' => 'Grandi',
			'param_VehicleFilters_CabinsNo' => 'Cabine',
			'param_VehicleFilters_Doors' => 'Porte',
			'param_VehicleFilters_FuelType' => 'Alimentazione',
			'param_VehicleFilters_FuelType_Diesel' => 'Diesel',
			'param_VehicleFilters_FuelType_Electric' => 'Elettrica',
			'param_VehicleFilters_FuelType_Gpl' => 'GPL',
			'param_VehicleFilters_FuelType_Hybrid' => 'Ibrida',
			'param_VehicleFilters_FuelType_Methane' => 'Metano',
			'param_VehicleFilters_FuelType_Petrol' => 'Benzina',
			'param_VehicleFilters_Gearshift_Automatic' => 'Autom.',
			'param_VehicleFilters_Gearshift_Manual' => 'Manuale',
			'param_VehicleFilters_KmLimit_None' => 'Km illimitati',
			'param_VehicleFilters_Seats' => 'Posti',
			'param_VehicleFilters_SeatsNo' => 'Pers. max.',
			'param_VehicleFilters_SmallBags' => 'Piccole',
			'param_VehicleFilters_ToiletsNo' => 'WC',

			'param_Description' => '',
			'param_Description_Text' => '',

			'param_VehicleDetails' => 'Caratteristiche',
			'param_VehicleDetails_Air' => 'Climatizzatore',
			'param_VehicleDetails_Air_0' => 'No',
			'param_VehicleDetails_Air_1' => 'Sì',
			'param_VehicleDetails_Doors' => 'Porte',
			'param_VehicleDetails_LiftingPlatform' => 'Pedana sollevatrice',
			'param_VehicleDetails_LiftingPlatform_0' => 'No',
			'param_VehicleDetails_LiftingPlatform_1' => 'Sì',
			'param_VehicleDetails_EmissionClass' => 'Euro',
			'param_VehicleDetails_EngineSize' => 'Cilindrata',
			'param_VehicleDetails_FuelType' => 'Alimentazione',
			'param_VehicleDetails_FuelType_Diesel' => 'Diesel',
			'param_VehicleDetails_FuelType_Electric' => 'Elettrica',
			'param_VehicleDetails_FuelType_Gpl' => 'GPL',
			'param_VehicleDetails_FuelType_Hybrid' => 'Ibrida',
			'param_VehicleDetails_FuelType_Methane' => 'Metano',
			'param_VehicleDetails_FuelType_Petrol' => 'Benzina',
			'param_VehicleDetails_Pax' => 'Passeggeri',
			'param_VehicleDetails_Radio' => 'Autoradio',
			'param_VehicleDetails_Radio_0' => 'No',
			'param_VehicleDetails_Radio_1' => 'Sì',
			'param_VehicleDetails_TankSize' => 'Capacità serbatoio',

			'param_InteriorDimensions' => 'Dimensioni interni',
			'param_InteriorDimensions_WeightLimit' => 'Portata',
			'param_InteriorDimensions_Length' => 'Lunghezza',
			'param_InteriorDimensions_Width' => 'Larghezza',
			'param_InteriorDimensions_WheelArchWidth' => 'Larghezza passaruote',
			'param_InteriorDimensions_Volume' => 'Volume',
			'param_InteriorDimensions_Height' => 'Altezza',

			'param_ExteriorDimensions' => 'Dimensioni esterne',
			'param_ExteriorDimensions_Wheelbase' => 'Passo',
			'param_ExteriorDimensions_Width' => 'Larghezza',
			'param_ExteriorDimensions_Length' => 'Lunghezza',
			'param_ExteriorDimensions_Height' => 'Altezza',

			'results_checkout' => 'Ritiro',
			'results_checkin' => 'Riconsegna',
			'result_day' => 'Giorno',
			'result_days' => 'Giorni',
			'results_details' => '⤼ Dettagli',
			'results_details_back' => '⤺ Indietro',
			'results_group_rate' => 'Da {rate}',
			'results_km_limit_day' => 'gg',
			'results_none' => 'Ci dispiace, ma non è stato trovato nessun risultato con i criteri di ricerca inseriti.',
			'results_optionals' => '* Suppl. {name}: {price}',
			'results_options_day' => 'Tariffa valida per <b>1 giorno</b>',
			'results_options_days' => 'Tariffa valida per <b>{days} giorni</b>',
			'results_options_hint' => 'Clicca sulla tariffa che fa per te per andare avanti.',
			'results_rate' => 'Prezzo',
			'results_rate_included' => 'Incluso',
			'results_remove' => 'Elimina',
			'results_required' => 'Obbligatorio',
			'results_select' => 'Scegli',
		),

		'en' =>
		[
			'error' => 'We\'re sorry, seems like the website is having some problems right now. Try to reload the page o try again later in case the problem persists.',

			'label_cancel' => 'Cancel',
			'label_loading' => 'Loading…',
			'label_modify' => 'Modify',
			'label_search' => 'Search',
			'label_select' => 'Select…',

			'otabo_settings' => 'Otabo settings',

			'otabo_auth' => 'Authentication',
			'otabo_uuid' => 'UUID',
			'otabo_uuid.desc' => 'Insert the agent/broker id as displayed on CRS.',
			'otabo_api_key' => 'API Key',
			'otabo_api_key.desc' => 'Insert the agent\'s or broker\'s API key exactly how it\'s displayed on CRS.',

			'otabo_payments' => 'Payments',
			'otabo_mipp_api_key' => 'MiPP API Key',
			'otabo_mipp_api_key.desc' => 'Insert the MiPP API key that My Appy has provided to you. Without a valid MiPP API key the Otabo only permits to save quotations whereas with MiPP you can actually receive payments and thus real reservations.',

			'otabo_hue' => 'H value of main color HSL',
			'otabo_sat' => 'S value of main color HSL',
			'otabo_lig' => 'L value of main color HSL',

			'otabo_custom_phrases' => 'Personalized strings',
			'otabo_custom_phrases.desc' => 'Use this field to personalize the strings used by Otabo. Contact My Appy to get to know more.<br /><b>Phrase mapping:</b><ul>
					<li>booking_base_{id}: Base names</li>
					<li>booking_clazz_{name}: Macrogroups</li>
					<li>booking_group_{name}: Collective groups</li>
					<li>booking_vehicle_{name}: Vehicle groups</li>
					<li>booking_tariff_{category name}: Tariff option</li>
					<li>booking_coverage_{id}_title: Coverage titles</li>
					<li>booking_coverage_{id}_desc: Coverage text</li>
					<li>booking_optionals_{group_id}: Optionals group id</li>
					<li>booking_optionals_{ota_eqp_id}_title: Optionals name</li>
					<li>booking_optionals_{ota_eqp_id}_desc: Optionals description</li>
				</ul>',
			'otabo_pages' => 'Pages',
			'otabo_form_url' => 'Search form',
			'otabo_form_url.desc' => 'Page that displays the main search form.',
			'otabo_results_url' => 'Results list',
			'otabo_results_url.desc' => 'Page that displays the search results.',
			'otabo_coverages_url' => 'Coverages',
			'otabo_coverages_url.desc' => 'Page that displays the coverage options.',
			'otabo_optionals_groups' => 'Optionals groups',
			'otabo_optionals_groups.desc' => 'Use this field to organize the optionals list. Contact My Appy to get to know more.',
			'otabo_optionals_url' => 'Optionals',
			'otabo_optionals_url.desc' => 'Page that displays the optionals.',
			'otabo_booking_url' => 'Personal information',
			'otabo_booking_url.desc' => 'Page that allows the user to insert his or her personal details in order to perform a booking.',
			'otabo_confirm_url' => 'Confirmation',
			'otabo_confirm_url.desc' => 'Page that displays the booking confirmation.',

			'otabo_params' => 'Parameters',
			'otabo_booking_fields' => 'Personal information fields',
			'otabo_booking_fields.desc' => 'Use this field to specify the visible fields on the page that lets the user enter his or her personal information. Contact My Appy to get to know more.',
			'otabo_booking_fields_mandatory' => 'Required personal information fields',
			'otabo_booking_fields_mandatory.desc' => 'Use this field to specify which of the fields above are mandatory. Contact My Appy to get to know more.',
			'otabo_booking_fields_invoicing' => 'Invoicing data fields',
			'otabo_booking_fields_invoicing.desc' => 'Activate to show a separate invoicing section where the user may insert additional information for invoicing purposes.',
			'otabo_form_bases' => 'Bases filter',
			'otabo_form_bases.desc' => 'List the bases to be shown separated by a comma or leave field empty to display all available bases.',
			'otabo_form_classes' => 'Display macro group tabs',
			'otabo_form_classes.desc' => 'Display a top bar above the form that allows the user to select the desired macro group to search for.',
			'otabo_form_classes_list' => 'Macro group filters',
			'otabo_form_classes_list.desc' => 'Use this field to filter e reorder the macro groups that are displayed above the search form. Write the group names exactly how they are displayed on CRS separating them by a comma (","). Leave the field empty if you would like to use the groups as they are registered with CRS.',
			'otabo_form_one_way_trips' => 'Display field to select different dropoff location',
			'otabo_form_one_way_trips.desc' => 'Use this field to visualize a field that lets the user select a different location for the vehicle dropoff.',
			'otabo_form_overflow_fix' => 'Overflow fix level',
			'otabo_form_overflow_fix.desc' => 'If you have the problem that the container that contains the otabo form cuts off elements from the form insert this field to indicate the number of levels that the plugin will try to override the elements\' overflow property (overflow=visible) for.',
			'otabo_form_young_driver' => 'Young driver flag',
			'otabo_form_young_driver.desc' => 'Set this flag if you want to display a checkbox on the form page to indicate that we\'re looking for cars for young drivers.',
			'otabo_result_details' => 'Display details',
			'otabo_result_details.desc' => 'If you want to display further details - apart from standard fields like number of seats, etc.; in the results list write the IDs of each section (which you can find as Otabo parameters on CRS under Tariffs > Web) here separating them with a comma (",").',
			'otabo_result_groups' => 'Display collective groups',
			'otabo_result_groups.desc' => 'Set this flag if you want to display collective groups above the results list.',
			'otabo_result_full_width' => '100% width display',
			'otabo_result_full_width.desc' => 'Set this flag if you want to display the results list at full width, i.e. one result on a separate line. In this case, the same vehicle will be displayed only once and different tariffs options will be listed in the result box on the right side to let the user make its choice.',
			'otabo_links' => 'Links',
			'otabo_terms_url' => 'Terms and conditions',
			'otabo_terms_url.desc' => 'Link that takes the user to the terms and conditions page.',
			'otabo_privacy_url' => 'Privacy policy',
			'otabo_privacy_url.desc' => 'Link that takes the user to the privacy policy details.',

			'otabo_footer' => '<h1>Elements to add to the pages</h1>
				<p>All the elements provided by Otabo con be added to any page by inserting its code tag as follows: <b>[element_name param_a="A" param_b="B"]</b>.
					Below you can find a list with all available elements and their parameters. If an element does not contain any parameters leave out the param_a="A", etc. part.</p>
				<dl>
					<dt>[otabo_form]</dt>
					<dd>Prints the main form that allows the user to choose the pickup and return locations and date and times.
					If the <b>Display macro group tabs</b> flag is set a tab bar with the different groups will be displayed above the form.</dd>

					<dt>[otabo_results]</dt>
					<dd>Prints the results list.</dd>

					<dt>[otabo_booking_coverages]</dt>
					<dd>Prints the coverage options list.</dd>

					<dt>[otabo_booking_optionals]</dt>
					<dd>Prints the optionals list.</dd>

					<dt>[otabo_booking_details]</dt>
					<dd>Prints the form that lets the user insert his or her personal details in order to perform the booking.</dd>

					<dt>[otabo_booking_no]</dt>
					<dd>This code tag simply prints the number of the saved quotation or the booking number. You can include it in any floating text that informs the user about the outcome of the operation,
					e.g. <i>Congratulations, your booking no. [otabo_booking_no] has been submitted successfully.</i></dd>

					<dt>[otabo_booking_summary]</dt>
					<dd>Generates a table that summarizes the current selection, i.e. the vehicle, any selected coverage and chosen optionals. Next to each record
					a link will be printed as well that allows the user to return to the appropriate page to modify the selection.</dd>
				</dl>',

			'booking_backToSearch' => 'Go back to search results',
			'booking_canceled' => 'Reservation {ref_id} cancelled',
			'booking_confirmed' => 'Reservation {ref_id} confirmed',
			'booking_coverage' => 'Coverage option',
			'booking_coverage_deposit' => 'Security deposit',
			'booking_coverage_excess' => 'Damage/Theft excess',
			'booking_coverage_excess_damage' => 'Damage excess',
			'booking_coverage_excess_theft' => 'Theft excess',
			'booking_coverage_view_terms' => 'Read the terms and conditions for more information',
			'booking_date' => 'When and where',
			'booking_error_email_not_valid' => 'The email address is not valid.',
			'booking_error_emails_not_equal' => 'The inserted email addresses are not the same.',
			'booking_error_fill_form' => 'Please complete the whole form in order to proceed.',
			'booking_error_terms' => 'You have to accept the terms and conditions and the privacy policy.',
			'booking_loading' => 'Loading…',
			'booking_optionals' => 'Optionals',
			'booking_optionals_next' => 'Continue',
			'booking_optionals_special' => 'Recommonded for you',
			'booking_optionals_children' => 'Travelling with children',
			'booking_optionals_other' => 'Other optionals',
			'booking_optionals_totals' => 'Rental totals',
			'booking_optionals_13_desc' => 'Add a navigation system to your rental to have access to up-to-date maps and routing information.',
			'booking_optionals_101_desc' => 'Share the driving experience… Add more drivers to your rental package so that they benefit from the same coverage as the main driver.',
			'booking_pay' => 'Pay now {totals}',
			'booking_privacy' => 'privacy policy',
			'booking_privacy_text' => 'I hereby authorise the processing of my personal data according to the {privacy} in compliance with the GDPR (UE Regulation 2016/679) and the D.LGS. 2018/101 and later modifications.',
			'booking_quote' => 'Block price of {totals}',
			'booking_reservation' => 'Book and pay now {totals}',
			'booking_reservation_error' => 'We\'re sorry! Either you\'ve canceled the reservation or your selected vehicle has just been booked in the meantime by someone else. Your payment has thus been canceled. Feel free to perform a new search to find something else.',
			'booking_reservation_submitting' => 'Confirming reservation',
			'booking_reservation_title' => 'Reservation #{ref_id}',
			'booking_summary' => 'Summary',
			'booking_terms' => 'terms and conditions',
			'booking_terms_text' => 'I wish to rent the indicated vehicle according to the {terms} and the selected tariff. I have read and understood the terms and the tariff and coverage options.',
			'booking_totals' => 'Totals',
			'booking_totals_desc' => 'Booking totals',
			'booking_vehicle' => 'Vehicle',
			'booking_young_driver' => 'The driver is younger than 21 years',

			'checkout_base' => 'Pickup location',
			'checkout_base_select' => 'Select the pickup location first…',
			'checkout_date' => 'Pickup date and time',
			'checkin_base' => 'Dropoff location',
			'checkin_base_select' => 'Leave the vehicle at another location',
			'checkin_date' => 'Dropoff date and time',

			'details_address' => 'Address',
			'details_date_of_birth' => 'Date of birth (DD-MM-YYYY)',
			'details_city' => 'City',
			'details_country' => 'Country',
			'details_email' => 'Email address',
			'details_email_confirm' => 'Email address (confirm)',
			'details_flight_no' => 'Flight no.',
			'details_name' => 'Name',
			'details_phone' => 'Phone number',
			'details_place_of_birth' => 'Place of birth',
			'details_province' => 'State/Province',
			'details_surname' => 'Surname',
			'details_tax_number' => 'Tax number',
			'details_zip_code' => 'Postal code',

			'details_invoicing' => 'Invoicing data',
			'details_invoicing_desc' => '',
			'details_invoicing_address' => 'Address',
			'details_invoicing_city' => 'City',
			'details_invoicing_company' => 'Company name',
			'details_invoicing_country' => 'Country',
			'details_invoicing_name' => 'Name',
			'details_invoicing_pec' => 'PEC',
			'details_invoicing_province' => 'State/Province',
			'details_invoicing_sdi_code' => 'SdI Code',
			'details_invoicing_surname' => 'Surname',
			'details_invoicing_tax_number' => 'Tax number',
			'details_invoicing_type' => 'Company/Private',
			'details_invoicing_type_business' => 'Company',
			'details_invoicing_type_one_man_business' => 'Sole trader',
			'details_invoicing_type_private' => 'Private',
			'details_invoicing_type_public_amm' => 'Public administration',
			'details_invoicing_vat_number' => 'VAT number',
			'details_invoicing_zip_code' => 'Postal code',

			'format.datetime' => 'd-m-Y H:i',

			'month_0' => 'January',
			'month_1' => 'February',
			'month_2' => 'March',
			'month_3' => 'April',
			'month_4' => 'May',
			'month_5' => 'June',
			'month_6' => 'July',
			'month_7' => 'August',
			'month_8' => 'September',
			'month_9' => 'October',
			'month_10' => 'November',
			'month_11' => 'December',

			'param_ChmCharacteristics' => 'Characteristics',
			'param_ChmCharacteristics_Engine' => 'Engine',
			'param_ChmCharacteristics_LengthOverall' => 'L.O.A.',
			'param_ChmCharacteristics_LengthOverall_Unit' => 'm',
			'param_ChmCharacteristics_TankSize' => 'Tank size',
			'param_ChmCharacteristics_TankSize_Unit' => 'L',
			'param_ChmCharacteristics_Type' => 'Type',
			'param_ChmCharacteristics_WaterTankSize' => 'Water tank',
			'param_ChmCharacteristics_WaterTankSize_Unit' => 'L',
			'param_ChmCharacteristics_Year' => 'Year',

			'param_ChmEquipment' => 'Equipment',
			'param_ChmEquipment_BowThruster' => 'Bow thruster',
			'param_ChmEquipment_BowThruster_1' => 'Yes',
			'param_ChmEquipment_BowThruster_0' => 'No',
			'param_ChmEquipment_Gps' => 'GPS',
			'param_ChmEquipment_Gps_bg_vulcan' => 'GPS B&G Vulcan',
			'param_ChmEquipment_Gps_gps_plotter' => 'GPS Plotter',
			'param_ChmEquipment_Gps_navionics' => 'iPad (Navionics App)',
			'param_ChmEquipment_Mainsail' => 'Mainsail',
			'param_ChmEquipment_Mainsail_furling' => 'Furling',
			'param_ChmEquipment_Mainsail_lazy_bag' => 'Lazy bag',
			'param_ChmEquipment_Teak' => 'Teak',
			'param_ChmEquipment_Teak_cockpit' => 'Cockpit',
			'param_ChmEquipment_Teak_full' => 'Full',
			'param_ChmEquipment_Teak_none' => 'None',

			'param_VehicleFilters_BerthsNo' => 'Berths',
			'param_VehicleFilters_BigBags' => 'Big',
			'param_VehicleFilters_CabinsNo' => 'Cabins',
			'param_VehicleFilters_Doors' => 'Doors',
			'param_VehicleFilters_FuelType' => 'Fuel',
			'param_VehicleFilters_FuelType_Diesel' => 'Diesel',
			'param_VehicleFilters_FuelType_Electric' => 'Electric',
			'param_VehicleFilters_FuelType_Gpl' => 'GPL',
			'param_VehicleFilters_FuelType_Hybrid' => 'Hybrid',
			'param_VehicleFilters_FuelType_Methane' => 'Methan',
			'param_VehicleFilters_FuelType_Petrol' => 'Petrol',
			'param_VehicleFilters_Gearshift_Automatic' => 'Autom.',
			'param_VehicleFilters_Gearshift_Manual' => 'Manual',
			'param_VehicleFilters_KmLimit_None' => 'Unlimited Km',
			'param_VehicleFilters_Seats' => 'Seats',
			'param_VehicleFilters_SeatsNo' => 'Pers. max.',
			'param_VehicleFilters_SmallBags' => 'Small',
			'param_VehicleFilters_ToiletsNo' => 'WC',

			'param_Description' => '',
			'param_Description_Text' => '',

			'param_VehicleDetails' => 'Characteristics',
			'param_VehicleDetails_Air' => 'Air conditioning',
			'param_VehicleDetails_Air_0' => 'No',
			'param_VehicleDetails_Air_1' => 'Yes',
			'param_VehicleDetails_Doors' => 'Doors',
			'param_VehicleDetails_LiftingPlatform' => 'Lifting platform',
			'param_VehicleDetails_LiftingPlatform_0' => 'No',
			'param_VehicleDetails_LiftingPlatform_1' => 'Yes',
			'param_VehicleDetails_EmissionClass' => 'Euro',
			'param_VehicleDetails_EngineSize' => 'Engine size',
			'param_VehicleDetails_FuelType' => 'Fuel',
			'param_VehicleDetails_FuelType_Diesel' => 'Diesel',
			'param_VehicleDetails_FuelType_Electric' => 'Electric',
			'param_VehicleDetails_FuelType_Gpl' => 'GPL',
			'param_VehicleDetails_FuelType_Hybrid' => 'Hybrid',
			'param_VehicleDetails_FuelType_Methane' => 'Methan',
			'param_VehicleDetails_FuelType_Petrol' => 'Petrol',
			'param_VehicleDetails_Pax' => 'Passengers',
			'param_VehicleDetails_Radio' => 'Radio',
			'param_VehicleDetails_Radio_0' => 'No',
			'param_VehicleDetails_Radio_1' => 'Yes',
			'param_VehicleDetails_TankSize' => 'Tank size',

			'param_InteriorDimensions' => 'Internal dimensions',
			'param_InteriorDimensions_WeightLimit' => 'Weight limit',
			'param_InteriorDimensions_Length' => 'Length',
			'param_InteriorDimensions_Width' => 'Width',
			'param_InteriorDimensions_WheelArchWidth' => 'Wheel arch width',
			'param_InteriorDimensions_Volume' => 'Volume',
			'param_InteriorDimensions_Height' => 'Height',

			'param_ExteriorDimensions' => 'Exterior dimensions',
			'param_ExteriorDimensions_Wheelbase' => 'Wheelbase',
			'param_ExteriorDimensions_Width' => 'Width',
			'param_ExteriorDimensions_Length' => 'Length',
			'param_ExteriorDimensions_Height' => 'Height',

			'results_checkout' => 'Pickup',
			'results_checkin' => 'Dropoff',
			'result_day' => 'Day',
			'result_days' => 'Days',
			'results_details' => '⤼ Details',
			'results_details_back' => '⤺ Back',
			'results_group_rate' => 'From {rate}',
			'results_km_limit_day' => 'day',
			'results_none' => 'We\'re sorry but we have not found any results matching your search criteria.',
			'results_optionals' => '* {name} supplement: {price}',
			'results_options_day' => 'Tariff valid for <b>1 day</b>',
			'results_options_days' => 'Tariff valid for <b>{days} days</b>',
			'results_options_hint' => 'Select your tariff option to proceed.',
			'results_rate' => 'Price',
			'results_rate_included' => 'Included',
			'results_remove' => 'Eliminate',
			'results_required' => 'Required',
			'results_select' => 'Select',
		]
	);
}

function register_session_new()
{
	if(!session_id())
	{
		session_start();
	}
}

add_action('init', 'register_session_new');

function otabo_init()
{
	$otabo = new Otabo(get_user_locale());

	add_action('wp_enqueue_scripts', array($otabo, 'addScripts'));
	add_action('wp_ajax_nopriv_oaj', array($otabo, 'ajax'));

	add_shortcode('otabo_booking_no', array($otabo, 'postBookingNo'));
	add_shortcode('otabo_booking_coverages', array($otabo, 'postBookingCoverages'));
	add_shortcode('otabo_booking_details', array($otabo, 'postBookingDetails'));
	add_shortcode('otabo_booking_optionals', array($otabo, 'postBookingOptionals'));
	add_shortcode('otabo_booking_summary', array($otabo, 'postBookingSummary'));
	add_shortcode('otabo_form', array($otabo, 'postForm'));
	add_shortcode('otabo_results', array($otabo, 'postResults'));

	add_action('admin_init', array($otabo, 'otabo_register_settings'));
	add_action('admin_menu', array($otabo, 'otabo_register_options_page'));

	// Settings link on plugin page.
	add_filter('plugin_action_links_otabo/otabo.php', 'nc_settings_link');
	function nc_settings_link($links)
	{
		// Build and escape the URL.
		$url = esc_url(add_query_arg('page', 'otabo', get_admin_url() . 'admin.php'));

		// Create the link.
		$settings_link = "<a href='$url'>" . __('Settings') . '</a>';

		// Adds the link to the end of the array.
		array_push($links, $settings_link);

		return $links;
	}
}

add_action('init', 'otabo_init');

?>
